#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::Aliases
#
# The purpose of this class is fairly simple:
#
#   - Give it a string such as "qt4r" and it will point to "qt4ruby".
#
# Presently, this class only uses the Aliases defined by the file
# "cookbook_aliases.yml".
#
# You can also query and obtain this information.
#
# It may be better to require other files than this class directly, such
# as 'cookbooks/module_methods/is_included.rb', since that file offers
# additional functionality, as well as make available class
# Cookbooks:Aliases anyway.
# =========================================================================== #
# require 'cookbooks/aliases/aliases.rb'
# =========================================================================== #
require 'cookbooks/base/prototype.rb'
require 'cookbooks/aliases/constants.rb'
require 'cookbooks/aliases/help.rb'
require 'cookbooks/aliases/initialize.rb'
require 'cookbooks/aliases/dataset.rb'
require 'cookbooks/aliases/menu.rb'
require 'cookbooks/aliases/reset.rb'
require 'cookbooks/aliases/run.rb'
require 'cookbooks/aliases/cookbook_aliases.rb'
require 'cookbooks/misc/available_cookbooks.rb'

module Cookbooks # === Cookbooks::Aliases

class Aliases < Base # We can not subclass from Base here < Base

  # ========================================================================= #
  # === n_registered_aliases?
  # ========================================================================= #
  def n_registered_aliases?
    @internal_hash[:raw_dataset].keys.size
  end; alias n_registered_programs? n_registered_aliases? # === n_registered_programs?
       alias n_entries?             n_registered_aliases? # === n_entries?

  # ========================================================================= #
  # === search_for_this_program?
  # ========================================================================= #
  def search_for_this_program?
    ihash?[:search_for_this_program]
  end; alias search_for_which_program? search_for_this_program? # === search_for_which_program?
       alias program?                  search_for_this_program? # === program?
       alias program_name?             search_for_this_program? # === program_name?
       alias this_program?             search_for_this_program? # === this_program?
       alias input?                    search_for_this_program? # === input?

  # ========================================================================= #
  # === set_search_for_this_program
  #
  # This setter method will search for the given program at hand.
  # ========================================================================= #
  def set_search_for_this_program(i = DEFAULT_PROGRAM_ENTRY)
    i = i.first if i.is_a? Array
    i = DEFAULT_PROGRAM_ENTRY if i.nil?
    i = i.to_s.dup # To ensure that it is never nil.
    # ======================================================================= #
    # Remove all '_' characters since as of March 2012.
    # ======================================================================= #
    i.delete!('-') if i.include? '-'
    ihash?[:search_for_this_program] = i
  end

  # ========================================================================= #
  # === set_keys
  #
  # This will keep track of how many keys there are in our dataset.
  # ========================================================================= #
  def set_keys(i = dataset?.keys.sort)
    @internal_hash[:keys] = i
  end

  # ========================================================================= #
  # === is_a_real_alias?
  #
  # This method tells us whether we have a real alias, that is originating
  # from the initial Hash, or whether we have an alias that we deduced.
  # ========================================================================= #
  def is_a_real_alias?(i = input?)
    raw_aliases?.has_key? i
  end

  # ========================================================================= #
  # === internal_hash?
  # ========================================================================= #
  def internal_hash?
    @internal_hash
  end; alias ihash? internal_hash? # === ihash?

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @internal_hash[:dataset]
  end; alias dataset dataset? # === dataset
       alias data?   dataset? # === data?
       alias data    dataset? # === data

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === report_whether_the_input_is_an_alias
  #
  # This method can be used to report whether the given input is an
  # alias or whether it is not.
  # ========================================================================= #
  def report_whether_the_input_is_an_alias(i = program?)
    if is_included?(i)
      opnn; e 'The program `'+sfancy(i)+'` is an alias, pointing '\
              'at the program `'+simp(points_at?)+'`.'
    else
      opnn; e 'The program `'+sfancy(i)+'` is NOT an alias.'
    end
  end

  # ========================================================================= #
  # === find
  #
  # find() will output in a verbose manner, regardless of the verbosity
  # level.
  # ========================================================================= #
  def find(i = program?)
    if is_included?(i)
      opnn; e '`'+sfancy(i)+'` matches towards `'+sfancy(points_at?(i))+'`.'
    else
      opnn; e '`'+sfancy(i)+'` does not match to anything.'
    end
  end; alias show find # === show

  # ========================================================================= #
  # === is_included?
  # ========================================================================= #
  def is_included?(i = program_name?)
    dataset?.has_key? i
  end; alias is_an_alias? is_included? # === is_an_alias?
       alias includes?    is_included? # === includes?
       alias found?       is_included? # === found?
       alias was_found?   is_included? # === was_found?

  # ========================================================================= #
  # === report
  # ========================================================================= #
  def report
    opnn; e 'The available aliases are:'
    opnn; e keys?.join(', ')
    opnn; e 'The available datasets are:'
    pp data?
    opnn; e 'The total size of all available aliases-keys is '+
             keys?.size.to_s+' entries.'
  end

  # ========================================================================= #
  # === deep_show
  #
  # This will show all matches. It is thought to be used in conjunction
  # with a query (and obviously, the variable @is_included must be set
  # for this to work.
  # ========================================================================= #
  def deep_show
    if is_included?
      opnn; e 'Additionally, also showing all aliases starting '+
              'with `'+sfancy(program?)+'*`.'
      all_matches = keys?.select {|key| key =~ /^#{program?}/ }
      e all_matches.sort.join(', ').split(N).map {|entry| '  '+entry }.join(N)
    end
  end; alias show_all_matches deep_show # === show_all_matches

  # ========================================================================= #
  # === points_at?
  #
  # Return the alias-pointer, without any additional checking whatsoever.
  #
  # Note that [] is also available as alias.
  # ========================================================================= #
  def points_at?(i = program_name?)
    dataset?[i]
  end; alias real_alias    points_at? # === real_alias
       alias points_to?    points_at? # === points_at?
       alias expanded_name points_at? # === expanded_name # Just an alias to the above.
       alias replace       points_at? # === replace
       alias value?        points_at? # === value?        # is an alias as well.
       alias []            points_at? # === []

  # ========================================================================= #
  # === show_size
  # ========================================================================= #
  def show_size
    opnn; e 'We have '+sfancy(n_registered_programs?.to_s)+
            ' unique programs registered.'
  end

  # ========================================================================= #
  # === avaiable_programs?
  # ========================================================================= #
  def available_programs?
    raw_aliases?.keys
  end; alias available_programs available_programs? # === available_programs
       alias programs?          available_programs? # === programs?
       alias aliases?           available_programs? # === aliases?

  # ========================================================================= #
  # === enable_aliases
  #
  # This method is required so that keys such as "qt4r" also work,
  # which is an alias of "qt4ruby" pointing towards "qt4qtruby".
  # ========================================================================= #
  def enable_aliases
    _ = {} # This will become the new Hash.
    # ======================================================================= #
    # Iterate over the old aliases.
    # ======================================================================= #
    dataset?.each_pair {|key, value|
      _[key]   = value
      _[value] = value # Also keep itself as name.
      loop {
        # =================================================================== #
        # Get all but the last two characters repeatedly, so we chop up the
        # keys.
        # =================================================================== #
        key = key[0..-2]
        _[key] = value
        break if key.empty?
      }
    }
    set_dataset(_)
  end

  # ========================================================================= #
  # === keys
  #
  # Assign the keys, then returns all keys. Will be sorted.
  # ========================================================================= #
  def keys?
    @internal_hash[:keys]
  end; alias keys keys? # === keys

  # ========================================================================= #
  # === show_all_available_entries
  # ========================================================================= #
  def show_all_available_entriesx
    pp keys?.sort
  end

  # ========================================================================= #
  # === translate_alias
  #
  # This method will try to "translate" an alias.
  # ========================================================================= #
  def translate_alias(i = program?)
    return dataset?[i] if is_included? # Found the result here.
    return i
  end; alias translate translate_alias # === translate
       alias value?    translate_alias # === value?
       alias []        translate_alias # === []
       alias result?   translate_alias # === result?
       alias result    translate_alias # === result

  # ========================================================================= #
  # === report_n_entries
  # ========================================================================= #
  def report_n_entries(i = n_entries?)
    e
    opnn; e 'We have n entries (== core aliases) registered:'
    opnn; e "  #{sfancy(i)}"
    e
  end

  # ========================================================================= #
  # === show_all_abbreviations
  # ========================================================================= #
  def show_all_abbreviations
    abbreviations = dataset?
    pp abbreviations
  end

end

# =========================================================================== #
# === Cookbooks.return_alias_program_name_for
#
# This method will return the (aliased) program name for the
# given input. If no alias name was found, it will return the
# input.
# =========================================================================== #
def self.return_alias_program_name_for(i)
  Cookbooks::Aliases.new(i).translate_alias
end

end

if __FILE__ == $PROGRAM_NAME
  require 'colour_e/autoinclude'
  require 'verbose_truth/autoinclude'
  _ = Cookbooks::Aliases.new(ARGV)
  if ARGV.empty?
    _.find 'glib2'
    _.report_n_entries
  else
    _.find(ARGV.first)
    # _.report_whether_the_input_is_an_alias
    # _.translate_alias # No output here.
    e 'We will next feedback information about '+simp(ARGV.first)+'.'
    e 'Is it a real alias? '+
      sfancy(VerboseTruth[_.is_a_real_alias?.to_s])
    e 'Is it an alias? '+
      sfancy(VerboseTruth[_.is_an_alias?.to_s])
    e 'It points towards '+
      sfancy(_.points_at?.to_s)+'.'
  end
end # ashower ruby
    # ashower qt4r
    # ashower ascii; ashower mencoder; ashower aclocal; ashower xfcem
    # ashower --n_entries