#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::Base
#
# This class will collect shared functionality for use in the other
# components of the Cookbooks project. They can subclass from it.
# =========================================================================== #
# require 'cookbooks/base/base.rb'; < ::Cookbooks::Base
# =========================================================================== #
require 'cookbooks/misc/save_file.rb'
require 'cookbooks/base/prototype.rb'
require 'cookbooks/base/ftp.rb'
require 'cookbooks/module_methods/show_overview_over_available_utility_scripts.rb'
require 'cookbooks/module_methods/is_included.rb'
require 'cookbooks/misc/available_cookbooks.rb'
require 'cookbooks/module_methods/fast_return_file_size_of_this_program.rb'

module Cookbooks

class Base

  # ========================================================================= #
  # Quite many class in the Cookbooks project make use of save-file
  # functionality, which explains the following include statement.
  # ========================================================================= #
  include ::Cookbooks::SaveFile

  # ========================================================================= #
  # === write_what_into
  # ========================================================================= #
  def write_what_into(what, into)
    Cookbooks::SaveFile.write_what_into(what, into)
  end; alias save_file write_what_into # === save_file

  # ========================================================================= #
  # === available_programs
  #
  # Feedback which programs are available, in sorted Array form.
  # ========================================================================= #
  def available_programs?
    Cookbooks.available_cookbooks?
  end; alias all_available_programs? available_programs? # === all_available_programs?
       alias available_programs      available_programs? # === available_programs
       alias available_cookbooks?    available_programs? # === available_cookbooks?
       alias available_cookbooks     available_programs? # === available_cookbooks

  # ========================================================================= #
  # === append_what_into
  # ========================================================================= #
  def append_what_into(what, into)
    Cookbooks::SaveFile.append_what_into(what, into)
  end

  # ========================================================================= #
  # === Cookbooks::Base.registered_binaries?
  #
  # Tell us where our binaries are registered.
  # ========================================================================= #
  def self.registered_binaries?
    Cookbooks.all_binaries?
  end

  # ========================================================================= #
  # === use_yaml_database_for_tags?
  # ========================================================================= #
  def use_yaml_database_for_tags?
    ::Cookbooks.use_yaml_database_for_tags?
  end

  # ========================================================================= #
  # === show_overview_over_available_utility_scripts
  # ========================================================================= #
  def show_overview_over_available_utility_scripts
    Cookbooks.show_overview_over_available_utility_scripts
  end

  # ========================================================================= #
  # === report_n_programs
  #
  # Usage example for the report functionality of this method:
  #   report_n_programs
  # ========================================================================= #
  def report_n_programs
    consider_reporting_how_many_programs_are_registered
  end

  # ========================================================================= #
  # === is_this_program_included?
  # ========================================================================= #
  def is_this_program_included?(i)
    ::Cookbooks.program_is_included?(i)
  end; alias included?              is_this_program_included? # === included?
       alias includes?              is_this_program_included? # === includes?
       alias program_was_found?     is_this_program_included? # === program_was_found?
       alias include?               is_this_program_included? # === include?
       alias is_included?           is_this_program_included? # === is_included?
       alias includes_this_program? is_this_program_included? # === includes_this_program?

  # ========================================================================= #
  # === gem_version
  #
  # This method is a wrapper over Gem::Version.new()
  #
  # The reason as to why this resides in a method is so that we can do
  # some input-sanitizing, and easier rescue, if this is necessary one
  # day.
  # ========================================================================= #
  def gem_version(i)
    i = i.to_s.delete('vr')
    begin
      if i =~ /\d+/ # Must have at the least one number.
        return Gem::Version.new(i)
      end
    rescue ArgumentError => error
      opnn; e 'An error occurred in gem_version()'
      pp error
    end
    nil # Indicate "failure" aka unable to parse this version.
  end

  # ========================================================================= #
  # === fast_return_file_size_of_this_program
  # ========================================================================= #
  def fast_return_file_size_of_this_program(i)
    ::Cookbooks.fast_return_file_size_of_this_program(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  p Cookbooks::Base
end