#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# Time and date-related content should be stored in this file here.
# =========================================================================== #
require 'time'
require 'cookbooks/module_methods/time.rb'

module Cookbooks

class Base

  # ========================================================================= #
  # === return_utc
  # ========================================================================= #
  def return_utc
    Time.now.getutc
  end

  # ========================================================================= #
  # === today?
  #
  # This method will return a String such as "21 September 2017".
  # ========================================================================= #
  def today?
    Time.now.strftime('%d %B %Y')
  end

  # ========================================================================= #
  # === get_date
  #
  # This method will return a String such as "21 Sep 2017".
  # ========================================================================= #
  def get_date
    ::Cookbooks.get_date
  end

  # ========================================================================= #
  # === return_date
  #
  # This method wil return a date such as "21.09.2017".
  # ========================================================================= #
  def return_date
    ::Cookbooks.return_date
  end

  # ========================================================================= #
  # === return_current_hour_minutes_second
  # ========================================================================= #
  def return_current_hour_minutes_second
    Time.now.strftime '%H:%M:%S'
  end

  # ========================================================================= #
  # === return_full_time
  #
  # This method will return a String such as '21.09.2017, 03:03:09'.
  # ========================================================================= #
  def return_full_time
    return_date+', '+return_current_hour_minutes_second
  end

  # ========================================================================= #
  # === return_weekday_based_on_utc
  #
  # This will return e. g. 'Mon' or 'Fri' or something like that.
  # ========================================================================= #
  def return_weekday_based_on_utc
    _ = return_utc
    wday = _.wday
    Time::RFC2822_DAY_NAME[wday]
  end

  # ========================================================================= #
  # === return_day_of_the_month_based_on_utc
  # ========================================================================= #
  def return_day_of_the_month_based_on_utc
    return_utc.day.to_s
  end

  # ========================================================================= #
  # === return_year_based_on_utc
  # ========================================================================= #
  def return_year_based_on_utc
    return_utc.year.to_s
  end

  # ========================================================================= #
  # === return_hours_minutes_seconds_based_on_utc
  # ========================================================================= #
  def return_hours_minutes_seconds_based_on_utc
    _ = return_utc
    _.strftime('%H:%M:%S') 
  end

  # ========================================================================= #
  # === return_utc_time_in_a_format_similar_to_slackware
  #
  # Slackware changelog uses a format such as this one here, in a UTC
  # format:
  #   Thu Sep 21 01:23:24 UTC 2017
  # ========================================================================= #
  def return_utc_time_in_a_format_similar_to_slackware
    return_weekday_based_on_utc.to_s+' '+
    return_month_based_on_utc.to_s+' '+
    return_day_of_the_month_based_on_utc+' '+
    return_hours_minutes_seconds_based_on_utc+
    ' UTC '+
    return_year_based_on_utc
  end
  
  # ========================================================================= #
  # === return_month_based_on_utc
  # ========================================================================= #
  def return_month_based_on_utc
    Date::MONTHNAMES[return_utc.month]
  end

  # ========================================================================= #
  # === return_month_based_on_this_number
  #
  # The input to this method should be an Integer, for the month, such
  # as 1,2,3 and so forth.
  # ========================================================================= #
  def return_month_based_on_this_number(i)
    Date::MONTHNAMES[i.to_i]
  end

end; end