# =========================================================================== #
# require 'cookbooks/class/constants.rb'
# =========================================================================== #
module Cookbooks

class Cookbook

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === DEFAULT_PROGRAM_VERSION
  # ========================================================================= #
  DEFAULT_PROGRAM_VERSION = DEFAULT_VERSION = '1.0.0'

  # ========================================================================= #
  # === REPLACE_PLUS_WITH_LONG_NAME
  #
  # If this constant is true then we will replace '+' with 'plus'
  # if they are part of a program name.
  # ========================================================================= #
  REPLACE_PLUS_WITH_LONG_NAME = false

  # ========================================================================= #
  # === Cookbooks::ARRAY_NIL_ENTRIES                                (nil tag)
  #
  # All entries that are nil by default.
  # ========================================================================= #
  ARRAY_NIL_ENTRIES = %w(
    short_description
    github
    set_env_variables
    homepage
  )

  # ========================================================================= #
  # === Cookbooks::ARRAY_ARRAY_ENTRIES                            (array tag)
  #
  # These entries should be kept as an Array.
  # ========================================================================= #
  ARRAY_ARRAY_ENTRIES = %w(
    binaries
    headers
    libraries
    postinstall
    pre_make_commands
    required_deps_on
    tags
    pkgconfig_files
  )

  # ========================================================================= #
  # === Cookbooks::ARRAY_EMPTY_VALUES
  #
  # All values that will initialize default to an empty string,
  # have to be registered here.
  # ========================================================================= #
  ARRAY_EMPTY_VALUES = %w(
    program_path
    program_full_name
    licence
    url1
    program_version
    configure_command
    use_this_make_command
    configure_base_dir
    optional_start_message
    preinstall
    make_options
    use_this_build_directory
    configure_options
    svn_url
    md5sum
  )

  # ========================================================================= #
  # === Cookbooks::ARRAY_MAIN_PROGRAM_ENTRIES
  #
  # We need the following 8 entries defined here, so that we do not
  # forget them lateron.
  #
  # These are the only entries we care about and describe the most
  # important components of a program.
  # ========================================================================= #
  ARRAY_MAIN_PROGRAM_ENTRIES = %w(
    archive_type
    base_dir
    program_full_name
    program_name
    program_path
    program_version
    real_short_name
    short_name
  )

  # ========================================================================= #
  # === Cookbooks.main_program_entries?
  # ========================================================================= #
  def self.main_program_entries?
    ARRAY_MAIN_PROGRAM_ENTRIES
  end

  # ========================================================================= #
  # === ARRAY_TRUE_VALUES                                          (true tag)
  #
  # Only a few variables are true for now, by default - prefix,
  # run_ldconfig and has_to_be_compiled.
  # ========================================================================= #
  ARRAY_TRUE_VALUES = %w(
    prefix
    run_ldconfig
    has_to_be_compiled
  )

  # ========================================================================= #
  # === Cookbooks::ARRAY_FALSE_VALUES                             (false tag)
  #
  # This constant collects all values that must be explicitely set to
  # false on startup.
  # ========================================================================= #
  ARRAY_FALSE_VALUES = %w(
    apply_patch
    build_static
    copy_source
    do_not_symlink
    do_not_download
    echo_yes
    enable_shared
    enable_static
    keep_extracted
    localstatedir
    override
    run_make_check
    use_autoconf
    use_autogen
    use_build_directory
    use_glib_schema
    use_this_make_install_command
    sed
    skip_configure
    symlink_glib_schema
    symlink_headers
    symlink_pkgconfig_files
  )

  # ========================================================================= #
  # === is_a_boolean_configuration_value?
  #
  # Query method to determine whether the input is a boolean configuration
  # value. For example, the String "use_glib_schema", would be a boolean
  # configuration value and thus this method would return true.
  # ========================================================================= #
  def is_a_boolean_configuration_value?(i)
    ( ARRAY_FALSE_VALUES + ARRAY_TRUE_VALUES ).flatten.include? i
  end

end; end