# =========================================================================== #
# require 'cookbooks/class/help.rb'
# =========================================================================== #
module Cookbooks

class Cookbook < Base

  # ========================================================================= #
  # === HOW_TO_INVOKE_THE_PROGRAM
  # ========================================================================= #
  HOW_TO_INVOKE_THE_PROGRAM = 'cookbook'

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # To invoke this particular method, try:
  #
  #   cookbook --help
  #
  # To invoke individual entries, do:
  #
  #   cookbook nprograms?
  #   cookbook --overview
  #   cookbook --version
  #
  # ========================================================================= #
  def show_help
    left_padding = '  ' # A bit of left-padding here.
    n_space_characters = 50
    how_to_invoke_the_program = HOW_TO_INVOKE_THE_PROGRAM
    opnn; e 'This class can feedback information about any '\
            'specific program.'
    e
    # ======================================================================= #
    # === Common usage examples
    # ======================================================================= #
    e 'Common usage examples:'
    e
    e left_padding+how_to_invoke_the_program+' ruby'
    e left_padding+how_to_invoke_the_program+' htop'
    e left_padding+how_to_invoke_the_program+' m4'
    e
    # ======================================================================= #
    # === Advanced Options
    # ======================================================================= #
    e 'Advanced options:'
    e
    e properly_pad_and_colourize_this_help_option('nprograms?',
        'show how many programs are registered in the cookbooks-collection'
      )
    e properly_pad_and_colourize_this_help_option('--aliases?',
        'show all available aliases to the registered cookbooks-programs'
      )
    e properly_pad_and_colourize_this_help_option('--available_programs?',
        'show all available programs'
      )
    e properly_pad_and_colourize_this_help_option('--batch-download-all-source-archives',
        'batch-download all source archives (optionally select for specific tags)'
      )
    e properly_pad_and_colourize_this_help_option('--overview',
        'give us an overview over all programs in the utility_scripts directory,'
      )
    e (' ' * n_space_characters)+'# of the cookbooks project'
    e properly_pad_and_colourize_this_help_option('--format=[name]',
        'feedback the format type of this entry,'
      )
    e (' ' * n_space_characters)+'# such as via '+simp('cookbook --format=postinstall')
    e properly_pad_and_colourize_this_help_option('--expand',
        'to expand the short yaml dataset into'
      )
    e properly_pad_and_colourize_this_help_option('--expand=PROGRAM_NAME',
        'to expand an individual program, such as python or ruby'
      )
    # ======================================================================= #
    # === --n-registered-binaries?
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--n-registered-binaries?',
        'to report how many registered binaries are available'
      )
    # ======================================================================= #
    # === --show-all-about=PROGRAM_NAME
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--show-all-about=PROGRAM_NAME',
        'to show all about the given program at hand'
      )
    e properly_pad_and_colourize_this_help_option('--permanently-disable-colours',
        'to permanently disable all colour output by the Cookbooks project'
      )
    e properly_pad_and_colourize_this_help_option('--update-kde-plasma',
        'batch-update (that is, download) all KDE5 components of KDE Plasma'
      )
    e (' ' * n_space_characters)+'# a long dataset; --initialize is an alias to this action'
    e properly_pad_and_colourize_this_help_option('--create-programs-version-html-file',
        'create and upload a .html file containing all '\
        'program_name-program_versions'
      )
    e (' ' * n_space_characters)+'# entries. This is primarily meant for my home system.'
    # ======================================================================= #
    # === --allowed_tags?
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--allowed_tags?',
       'show a list of all allowed tags in the '+simp('tags: subsection')+
       ' of a cookbook-yaml file'
      )
    # ======================================================================= #
    # === --generate_version_file
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--generate_version_file',
        'to generate a programs-version yaml file'
      )
    # ======================================================================= #
    # === --use-psych
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--use-psych',
        'to use psych for the yaml files'
      )
    # ======================================================================= #
    # === --use-syck
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--use-syck',
        'to use syck for the yaml files'
      )
    # ======================================================================= #
    # === --create-html-page-for-these-programs=
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--create-html-page-for-these-programs=',
        'to generate a .html page for the given programs; separate '\
        'the input via'
      )
    e (' ' * n_space_characters)+'# "'+sfancy(',')+'" such as in '+
      simp('=ruby,python,php')
    # ======================================================================= #
    # === --n_binaries?
    # ======================================================================= #
    e properly_pad_and_colourize_this_help_option('--n_binaries?',
        'tell us how many binaries/executables are tracked '\
        'in all cookbook-yaml files'
      )
    e properly_pad_and_colourize_this_help_option('--publish',
        'publish the list of all program versions of the cookbooks project'
      )
    e properly_pad_and_colourize_this_help_option('--total-size?',
        'show the total size of all locally available archives'
      )
    e properly_pad_and_colourize_this_help_option('--tutorial',
        'do show the short tutorial how the Cookbooks project may be used'
      )
    e properly_pad_and_colourize_this_help_option('--update-tags',
        'do update the tags defined in the various cookbooks'
      )
    e properly_pad_and_colourize_this_help_option('--validate-entries',
        'validate all Cookbook entries'
      )
    e properly_pad_and_colourize_this_help_option('--version',
        'feedback the current version of the Cookbooks gem'
      )
    e properly_pad_and_colourize_this_help_option('--set_source_dir=/tmp',
        'permanently set a new source dir (aka where all '\
        'archives are kept locally)'
      )
    e properly_pad_and_colourize_this_help_option('--set_temp_dir=/tmp',
        'permanently change the yaml file to set a '\
        'new temp directory'
      )
    e
  end

  # ========================================================================= #
  # === properly_pad_and_colourize_this_help_option
  # ========================================================================= #
  def properly_pad_and_colourize_this_help_option(
      key_to_invoke_the_method,
      text_that_is_to_be_displayed
    )
    left_padding = '  ' # A bit of left-padding here.
    n_left_pad_value  = 38
    n_left_pad_value -= key_to_invoke_the_method.size
    left_padding+HOW_TO_INVOKE_THE_PROGRAM+
                 skyblue(' '+key_to_invoke_the_method)+
                 (' ' * n_left_pad_value)+' # '+
                 text_that_is_to_be_displayed
  end

end; end