#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/cookbook/menu.rb'
# =========================================================================== #
require 'cookbooks/version/version.rb'
require 'cookbooks/class/help.rb'
require 'cookbooks/class/tutorial.rb'
require 'cookbooks/class/feedback/feedback_format_specification_of.rb'
require 'cookbooks/class/feedback/feedback_all_available_formats.rb'
require 'cookbooks/colours/colours.rb'
require 'cookbooks/module_methods/create_programs_version_html_file.rb'
require 'cookbooks/module_methods/report_how_many_binaries_are_registered.rb'
require 'cookbooks/module_methods/report_total_size_of_all_archives.rb'
require 'cookbooks/module_methods/show_last_configure_option.rb'
require 'cookbooks/module_methods/show_available_utility_scripts.rb'

module Cookbooks

class Cookbook < Base

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      # ===================================================================== #
      # Next, we will document some extra commands for the cookbooks
      # main executable. Only a few of them will be shown though.
      # ===================================================================== #
      case i # case tag
      # ===================================================================== #
      # === cookbooks --generate-homepage
      # ===================================================================== #
      when /^-?-?generate(_|-)?homepage$/,
           /--homepage$/,
           'genhomepage'
        require 'cookbooks/utility_scripts/generate_homepage/generate_homepage.rb'
        Cookbooks::GenerateHomepage.new
        exit
      # ===================================================================== #
      # === cookbooks --kde-status
      # ===================================================================== #
      when /^-?-?kde(_|-)?status$/i,
           /^-?-?report(_|-)?the(_|-)?kde(_|-)?status$/i
        report_the_kde_status
        exit
      # ===================================================================== #
      # === cookbooks --available-tags
      # ===================================================================== #
      when /^-?-?available(_|-)?tags$/i
        require 'cookbooks/module_methods/show_available_tags.rb'
        opnn; Cookbooks.show_available_tags(:default, :be_quiet)
        exit
      # ===================================================================== #
      # === cookbooks --download-all-source-archives
      # ===================================================================== #
      when /^-?-?download(_|-)?all(_|-)?source(_|-)?archives$/i,
           /^-?-?download(_|-)?source(_|-)?archives$/i,
           /^-?-?download(_|-)?sources$/i,
           /^-?-?grab(_|-)?source(_|-)?archives$/i
        unless Cookbooks.const_defined? :DownloadAllSourceArchives
          require 'cookbooks/module_methods/download_all_source_archives.rb'
        end
        Cookbooks.download_all_source_archives
        exit
      # ===================================================================== #
      # === cookbooks --update_all_ruby_gems
      # ===================================================================== #
      when /^-?-?update(_|-)?all(_|-)?ruby(_|-)?gems$/,
           /^-?-?update(_|-)?ruby(_|-)?gems$/
        require 'cookbooks/module_methods/update_all_ruby_gems.rb'
        Cookbooks.update_all_ruby_gems
        exit
      # ===================================================================== #
      # === cookbooks --update-kde-applications
      # ===================================================================== #
      when /^-?-?update(_|-)?kde(_|-)?applications$/i,
           /^-?-?download(_|-)?kde(_|-)?applications$/,
           /^-?-?kde(_|-)?apps$/
        update_kde_applications
        exit
      # ===================================================================== #
      # === cookbooks --n_binaries?
      # ===================================================================== #
      when /^-?-?n(_|-)?binaries\??$/,
           /^-?-?n(_|-)?registered(_|-)?binaries?\??$/
        Cookbooks.report_how_many_binaries_are_registered
        exit
      # ===================================================================== #
      # === cookbooks --show-all-about=htop
      # ===================================================================== #
      when /^-?-?show(_|-)?all(_|-)?about=(.+)/i,
           /^-?-?show(_|-)?all=(.+)/i,
           /^-?-?show(_|-)?file=(.+)/i
        show_this_program = $2.to_s.dup # This is valid for two entry points ^^^.
        show_this_program = $3.to_s.dup if $3
        show_all_about(show_this_program)
        exit
      # ===================================================================== #
      # === cookbooks --update-mate-desktop
      # ===================================================================== #
      when /^-?-?update(_|-)?mate(_|-)?desktop$/i,
           /^-?-?update(_|-)?mate$/i,
           /^-?-?mate(_|-)?desktop$/i,
           /^-?-?check(_|-)?for(_|-)?mate(_|-)?updates$/i,
           /^-?-?check(_|-)?for(_|-)?mate(_|-)?desktop(_|-)?updates$/i,
           /^-?-?check(_|-)?for(_|-)?mate(_|-)?desktop$/i
        update_mate_desktop_components
        exit
      # ===================================================================== #
      # === cookbooks --permanently-disable-colours
      # ===================================================================== #
      when /^-?-?permanently(_|-)?disable(_|-)?colours/
        ::Cookbooks.permanently_disable_colours
        exit
      # ===================================================================== #
      # === cookbooks --nocolours
      # ===================================================================== #
      when /^-?-?nocolours/,
           /^-?-?nocol/,
           /^-?-?disable_?colours/,
           'NOCOL','NOCOLOURS'
        disable_colours
      # ===================================================================== #
      # === cookbooks --batch-validate
      # ===================================================================== #
      when /^-?-?batch(_|-)?validate$/,
           /^-?-?validate(_|-)?entries$/,
           /^-?-?validate$/
        batch_validate_the_cookbooks
        exit
      # ===================================================================== #
      # === cookbooks --expand
      # ===================================================================== #
      when /^-?-?expand$/,
           /^-?-?expand-?dataset$/,
           /^-?-?initialize$/,
           'expand',
           'expanddataset'
        unless Cookbooks.const_defined? :ExpandCookbooks
          require 'cookbooks/utility_scripts/expand_cookbooks.rb'
        end
        Cookbooks::ExpandCookbooks.new
        exit
      # ===================================================================== #
      # === cookbooks --expand
      # ===================================================================== #
      when /^-?-?expand=(.+)$/,
           /^-?-?expand-?dataset=(.+)$/
        unless Cookbooks.const_defined? :ExpandCookbooks
          require 'cookbooks/utility_scripts/expand_cookbooks.rb'
        end
        Cookbooks::ExpandCookbooks.new($1.to_s.dup)
        exit
      # ===================================================================== #
      # === cookbooks --generate-version-file
      # ===================================================================== #
      when /^-?-?generate(_|-)?version(_|-)?file/,
           /^-?-?gversionfile/
        require 'cookbooks/module_methods/generate_programs_version_yaml_file.rb'
        Cookbooks.generate_programs_version_yaml_file
        exit
      # ===================================================================== #
      # === cookbooks --update-kde-porting-aids
      # ===================================================================== #
      when /^-?-?update(_|-)?kde(_|-)?porting(_|-)?aids$/i
        update_kde_porting_aids
        exit
      # ===================================================================== #
      # === cookbooks --update-kde-framework
      # ===================================================================== #
      when /^-?-?update(_|-)?kde(_|-)?framework$/i
        update_kde_framework
        exit
      # ===================================================================== #
      # === cookbooks --update-kde-plasma
      # ===================================================================== #
      when /^-?-?update(_|-)?kde(_|-)?plasma$/i
        update_kde_plasma
        exit
      # ===================================================================== #
      # === cookbooks --use-syck
      # ===================================================================== #
      when /^-?-?use(_|-)?syck$/i,
           /^-?-?permanently(_|-)?use(_|-)?syck$/i
        do_permanently_use_syck
        exit
      # ===================================================================== #
      # === cookbooks --use-psych
      # ===================================================================== #
      when /^-?-?use(_|-)?psych$/i,
           /^-?-?permanently(_|-)?use(_|-)?psych$/i
        do_permanently_use_psych
        exit
      # ===================================================================== #
      # === cookbooks --create-html-page-for-these-programs=ruby,python,tcl,filelight
      # ===================================================================== #
      when /^-?-?create(_|-)?html(_|-)?page(_|-)?for(_|-)?these(_|-)?programs=(.+)/
        these_programs = $6.to_s.dup
        if these_programs.include? ','
          these_programs = these_programs.split(',').flatten
        end
        require 'cookbooks/module_methods/create_html_page_for_these_programs.rb'
        Cookbooks.create_html_page_for_these_programs(these_programs)
        exit
      # ===================================================================== #
      # === cookbooks --create_program_version_url_file
      # ===================================================================== #
      when /^-?-?create(_|-)?program(_|-)?version(_|-)?url(_|-)?file$/
        require 'cookbooks/module_methods/create_program_version_url_file.rb'
        Cookbooks.create_program_version_url_file
        exit
      # ===================================================================== #
      # === cookbooks --publish-list-of-all-programs-version
      # ===================================================================== #
      when /^publish(_|-)?list(_|-)?of(_|-)?/i,
           /^-?-?publish/
        publish_list_of_all_programs_version # in file base/misc.rb
        exit
      # ===================================================================== #
      # === cookbooks --create-html-file
      # ===================================================================== #
      when /^-?-?create(_|-)?html(_|-)?file$/i,
           /^-?-?create(_|-)?programs(_|-)?version(_|-)?html(_|-)?file$/i,
           /^-?-?program(_|-)?versions$/i
        Cookbooks.create_programs_version_html_file
        exit
      # ===================================================================== #
      # === cookbooks --path_to_source_directory=
      # ===================================================================== #
      when /^-?-?path(_|-)?to(_|-)?source(_|-)?directory=(.+)$/, # for $4
           /^-?-?set(_|-)?archive(_|-)?directory=(.+)$/,         # for $3
           /^-?-?source(_|-)?directory=(.+)$/                    # for $2
        # =================================================================== #
        # Usage examples:
        #   cookbooks --path_to_source_directory=/Depot/Files
        #   cookbooks --set-archive-directory=/opt2
        #   cookbooks --source_directory=/Depot/Files
        # =================================================================== #
        if $4
          _ = $4.to_s.dup
        elsif $3
          _ = $3.to_s.dup
        else
          _ = $2.to_s.dup
        end # ^^^ we have to work through the special $ variables.
        ::Cookbooks.permanently_set_path_to_this_source_directory(_)
        exit
      # ===================================================================== #
      # === cookbooks --total-size?
      # ===================================================================== #
      when /^-?-?total(_|-)?size\??$/,
           /^-?-?report(_|-)?total(_|-)?size(_|-)?of(_|-)?all(_|-)?archives$/
        Cookbooks.report_total_size_of_all_archives
        exit
      # ===================================================================== #
      # === cookbooks --base_directory?
      # ===================================================================== #
      when /^-?-?base(_|-)?directory\??$/,
           /^-?-?archive(_|-)?directory\??$/
        e 'The directory for source archives can be found at: '
        e
        e '  '+sdir(Cookbooks.source_directory?)
        e
        exit
      # ===================================================================== #
      # === cookbooks --path_to_source_directory?
      # ===================================================================== #
      when /^-?-?path(_|-)?to(_|-)?source(_|-)?directory\??$/,
           /^-?-?path\??$/
        show_path_to_the_source_directory_yaml_file
        exit
      # ===================================================================== #
      # === cookbooks --allowed_tags?
      # ===================================================================== #
      when /^-?-?allowed(_|-)?tags\??/,
           /^-?-?registered(_|-)?tags\??/,
           /^valid(_|-)?tags\??/,
           /^-?-?valid(_|-)?tags\??/
        load_and_display_the_allowed_tags
        exit
      # ===================================================================== #
      # === cookbook --aliases?
      # ===================================================================== #
      when /^-?-?aliases\??/
        show_available_aliases
        exit
      # ===================================================================== #
      # === cookbooks --overview
      # ===================================================================== #
      when /^-?-?overview$/,'OVERVIEW' # cookbooks2 OVERVIEW
        show_overview_over_available_utility_scripts
        exit
      # ===================================================================== #
      # === cookbooks --total-size-of-locally-available-files
      # ===================================================================== #
      when /^-?-?total(_|-)?size(_|-)?of(_|-)?locally(_|-)?available(_|-)?files$/
        report_total_size_of_locally_available_files
        exit
      # ===================================================================== #
      # === cookbooks --scripts?
      # ===================================================================== #
      when /^-?-?scripts\??$/
        show_available_utility_scripts
      # ===================================================================== #
      # === cookbooks --help
      # ===================================================================== #
      when /^-?-?help$/,
           /-?-?HELP/ # cookbooks2 --help
        show_help
        exit
      # ===================================================================== #
      # === cookbook --update-tags
      # ===================================================================== #
      when /^-?-?update(-|_)?tags$/
        update_tags
        exit
      # ===================================================================== #
      # === cookbooks --show-last-updated-programs
      #
      # Show the last n updated programs, in a table-layout. This
      # presently defaults to show ALL the available programs.
      # ===================================================================== #
      when /^-?-?show(-|_)?last(-|_)?updated(-|_)?programs/,
           /^-?-?last(-|_)?updated\??/, # cookbooks --last-updated?
           'slu'
        Cookbooks::ShowLastUpdated.new('-n '+n_programs_available?.to_s)
        exit
      # ===================================================================== #
      # === cookbooks --feedback_all_available_formats
      # ===================================================================== #
      when /^-?-?feedback(_|-)?all(_|-)?available(_|-)?formats/
        Cookbooks::Cookbook.feedback_all_available_formats
        exit
      # ===================================================================== #
      # === cookbooks --check-for-gnome-updates
      # ===================================================================== #
      when /^-?-?check(_|-)?for(_|-)?gnome(_|-)?updates$/,
           /^-?-?gnome$/
        require 'cookbooks/utility_scripts/check_for_gnome_updates.rb'
        Cookbooks::CheckForGnomeUpdates.new
        exit
      # ===================================================================== #
      # === cookbooks --version?
      # ===================================================================== #
      when /^-?-?version\??$/,'VERSION','-version'
        e 'The current Cookbooks version is: '+simp(Cookbooks.version?)
        exit
      # ===================================================================== #
      # === scookie htop --array
      # ===================================================================== #
      when /-?-?array$/
        output_main_dataset
        exit
      # ===================================================================== #
      # === cookbooks --format=preinstall
      # === cookbooks --f=enable_shared
      # ===================================================================== #
      when /^-?-?format=(.+)/,
           /^-?-?f=(.+)/
        feedback_format_specification_of($1.to_s.dup)
        exit
      # ===================================================================== #
      # === cookbooks --set_source_dir=/opt
      # ===================================================================== #
      when /^-?-?set(_|-)?source(_|-)?dir=(.+)$/,
           /^-?-?set(_|-)?src(_|-)?dir=(.+)$/,
           /^-?-?permanently(_|-)?set(_|-)?base(_|-)?directory(_|-)?to=(.+)$/
        _ = $3.to_s.dup
        _ = $5.to_s.dup if $5 # This is for the variant on the third line, see: http://rubular.com/r/6E6v45rB7u
        unless _.end_with? '/'
          _ = _.dup if _.frozen?
          _ << '/' unless _.end_with? '/'
        end
        opnn; e 'We will permanently set to a new source '\
                'directory next ('+simp(_)+')'
        Cookbooks.permanently_set_source_directory(_)
        exit
      # ===================================================================== #
      # === cookbooks --debug
      # ===================================================================== #
      when /-?-?debug$/
        enable_debug
      # ===================================================================== #
      # === cookbooks --show-available-utility-scripts
      # ===================================================================== #
      when /-?-?show-?available-?utility-?scripts$/,
           /-?-?show-?available-?scripts$/
        Cookbooks.show_available_utility_scripts
        exit
      # ===================================================================== #
      # === cookbooks --correct_dependencies?
      # ===================================================================== #
      when /^-?-?correct(_|-)?dependencies\??$/,
           'DEPS','--cdeps?'
        Cookbooks::CheckForCorrectDependencies[]
        exit
      # ===================================================================== #
      # === cookbooks --vca
      # ===================================================================== #
      when /^-?-?vca$/
        Cookbooks::ValidateCookbookAliases.new
        exit
      # ===================================================================== #
      # === cookbooks --format=postinstall
      # ===================================================================== #
      when /^format=(.+)$/,
           /^f=(.+)/
        feedback_format_specification_of($1.to_s.dup)
        exit
      # ===================================================================== #
      # === cookbooks --nprograms
      # ===================================================================== #
      when /^-?-?n(_|-)?programs\??/,
           'registered_programs?',
           'nregistered?'
        show_n_programs_available
        exit
      # ===================================================================== #
      # === cookbooks --size
      # ===================================================================== #
      when /^-?-?size$/ # cookbook --size "php,ruby,perl,python"
        if i.include? '=' # Assume input such as this one here: "--size=php,ruby,perl,python"
          report_size_of_these_programs(i)
        end
        exit
      # ===================================================================== #
      # === cookbooks --set_temp_dir=/tmp
      # ===================================================================== #
      when /-?-?set(_|-)?temp(_|-)?dir=(.+)/
        _ = $3.to_s.dup
        opnn; e 'We will permanently set to a new temp '\
                'directory next ('+simp(_)+')'
        Cookbooks.permanently_set_temp_directory(_)
        exit
      # ===================================================================== #
      # === cookbooks --allowed_entries?
      # ===================================================================== #
      when '--allowed_entries?','allowed_entries?','--allowed_entries',
           '--allowed-cookbook-entries'
        Cookbooks::AllowedCookbookEntries.new
        exit
      # ===================================================================== #
      # === cookbooks --scanu
      # ===================================================================== # 
      when /^-?-?scanu/
        Cookbooks::ScanForMissingLastUpdateEntries.new
        exit
      # ===================================================================== #
      # === cookbooks allurls
      # ===================================================================== #
      when /^-?-?all-?urls$/,'--aurls'
        Cookbooks::AllUrls.new
        exit
      # ===================================================================== #
      # === cookbooks --return_n_programs_available
      # ===================================================================== #
      when /^-?-?return(_|-)?n(_|-)?programs(_|-)?available/
        return_n_programs_available
      # ===================================================================== #
      # === cookbooks --last_option?
      # ===================================================================== #
      when /^-?-?last(_|-)?option\?/,'show_last_configuration_options',
           'lastoptions?','lastoptions','loptions'
        ::Cookbooks.show_last_configure_option
        exit
      # ===================================================================== #
      # === cookbooks --show_config
      # ===================================================================== #
      when /^-?-?show_?config/,'sconfig'
        show_configuration_options
      # ===================================================================== #
      # === cookbooks --available_programs?
      # ===================================================================== #
      when /^-?-?available(_|-)?programs\??/i
        show_all_available_program
        exit
      # ===================================================================== #
      # === cookbooks --tutorial
      # ===================================================================== #
      when /^-?-?tutorial/
        ::Cookbooks.show_tutorial
        exit
      end
    end
  end; alias check_against_menu menu # === check_against_menu

end; end