#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
#   (1) real_short_name      like: 'gnomemahjonng'  # This is like short_name but without any possible '-'
#   (2) short_name           like: 'gnome-mahjonng' # This may invoke (1) and may include '-' characters.   
#   (3) program_version      like: '5.08'
#   (4) archive_type         like: '.tar.bz2' # Yes, must have a leading '.'
#   (5) base_dir             like: '/Users/x/SRC/gnomemahjongg/'
#   (6) program_name         like: 'gnome-mahjonng-5.08' # This one includes the version. It is short_name+'-'+program_version
#   (7) program_full_name    like: 'gnome-mahjonng-5.08.tar.xz', so this is program_name?+archive_type?
#   (8) program_path         like: '/Users/x/SRC/gnomemahjongg/gnome-mahjonng-5.08.tar.xz' # base_dir?+program_full_name?
#   (9) program_compile_name like: "GNOME-Mahjonng". This can be independent of the other settings above.
# =========================================================================== #
require 'cookbooks/constants/archives.rb'

module Cookbooks

class Cookbook < Base

  # ========================================================================= #
  # === set_in_yaml_dataset                             (set tag, setter tag)
  #
  # This is the main setter for the yaml dataset.
  #
  # Note that you can also use it to set several subsequent entries.
  #
  # We will also batch-update if you send in a String that contains ','.
  #
  # An example for this would be the following:
  #   set '4,5,6,7,8'
  # ========================================================================= #
  def set_in_yaml_dataset(
      key,
      new_value = nil
    )
    if key.is_a? Array # Recursive invocation in this case.
      key.flatten.each {|inner_key|
        set(inner_key, new_value)
      }
    elsif key.is_a?(String) and key.include?(',')
      key.split(',').each {|inner_entry|
        set(inner_entry, new_value) # Recursive calling then.
      }
    elsif key.is_a?(String) and key.include?('-') # Ok, we have a range here such as 2-5, meaning to update 2,3,4,5.
      splitted = key.split('-')
      min, max = splitted[0].to_i, splitted[1].to_i
      min.upto(max) {|entry| set(entry) }
    else
      # ===================================================================== #
      # All keys in the yaml_dataset have to be kept as Strings, for now.
      # This may change one day but for now, it will remain how it is.
      # ===================================================================== #
      key = key.to_s unless key.is_a? String
      # ===================================================================== #
      # After this point, all keys will be Strings. We must be careful when
      # we apply rds() - valid URLs may contain '//'.
      # ===================================================================== #
      if new_value and new_value.is_a? String # Only sanitize Strings.
        if new_value.include?('//') and
          !new_value.include?('http://')
          new_value = rds(new_value)
        end
      end
      # ===================================================================== #
      # Next set some aliases for keys issued.
      # ===================================================================== #
      case key
      when '1'
        key = 'real_short_name'
      when '2'
        key = 'short_name'
      when '3','version',/program_?version/
        key = 'program_version'
      when '4'
        key = 'archive_type'
      when '5','dir'
        key = 'base_dir'
      when '6'
        key = 'program_name'
      when '7'
        key = 'program_full_name'
      when '8','program_full_path'
        key = 'program_path'
      end
      # ===================================================================== #
      # Store in the internal Hash. The internal Hash must always exist,
      # for this to work.
      # ===================================================================== #
      @internal_hash[:yaml_dataset][key] = new_value
    end
  end; alias set              set_in_yaml_dataset # === set
       alias set_in_dataset   set_in_yaml_dataset # === set_in_dataset
       alias update_entries   set_in_yaml_dataset # === update_entries
       alias update_entry     set_in_yaml_dataset # === update_entry
       alias []=              set_in_yaml_dataset # === []=
       alias set_in_main_hash set_in_yaml_dataset

  # ========================================================================= #
  # === set_sed
  # ========================================================================= #
  def set_sed(i)
    set(:sed, i)
  end; alias sed= set_sed # === sed=

  # ========================================================================= #
  # === set_symlink_pkgconfig_files
  # ========================================================================= #
  def set_symlink_pkgconfig_files(i = true)
    set(:symlink_pkgconfig_files, i)
  end; alias symlink_pkgconfig_files= set_symlink_pkgconfig_files # === symlink_pkgconfig_files=

  # ========================================================================= #
  # === set_preinstall
  # ========================================================================= #
  def set_preinstall(i)
    set(:preinstall, i)
  end

  # ========================================================================= #
  # === set_postinstall
  # ========================================================================= #
  def set_postinstall(i)
    set(:postinstall, i)
  end

  # ========================================================================= #
  # === set_use_this_build_directory
  # ========================================================================= #
  def set_use_this_build_directory(i)
    set(:use_this_build_directory, i)
  end; alias use_this_build_directory= set_use_this_build_directory # === use_this_build_directory=

  # ========================================================================= #
  # === set_binaries
  # ========================================================================= #
  def set_binaries(i)
    set(:binaries, i)
  end

  # ========================================================================= #
  # === set_libraries
  # ========================================================================= #
  def set_libraries(i)
    set(:libraries, i)
  end

  # ========================================================================= #
  # === set_tags
  # ========================================================================= #
  def set_tags(i)
    set(:tags, i)
  end

  # ========================================================================= #
  # === set_use_build_directory
  # ========================================================================= #
  def set_use_build_directory(i)
    set(:use_build_directory, i)
  end; alias use_build_directory= set_use_build_directory # === use_build_directory=

  # ========================================================================= #
  # === set_configure_base_dir
  # ========================================================================= #
  def set_configure_base_dir(i)
    set(:configure_base_dir, i)
  end

  # ========================================================================= #
  # === set_symlink_headers
  #
  # If we want to symlink the headers, this is the setter-method that should
  # be called.
  # ========================================================================= #
  def set_symlink_headers(i = true)
    set(:symlink_headers, i)
  end; alias symlink_headers= set_symlink_headers # === symlink_headers=

  # ========================================================================= #
  # === set_pkgconfig_files
  # ========================================================================= #
  def set_pkgconfig_files(i)
    unless i.is_a? Array
      i = [i]
    end
    set(:pkgconfig_files, i)
  end; alias pkgconfig_files= set_pkgconfig_files # === pkgconfig_files?

  # ========================================================================= #
  # === set_use_glib_schema
  # ========================================================================= #
  def set_use_glib_schema(i = nil)
    if i
      set(:use_glib_schema, i)
    end
  end; alias use_glib_schema= set_use_glib_schema # === use_glib_schema=

  # ========================================================================= #
  # === set_autoconf
  # ========================================================================= #
  def set_autoconf(i)
    set(:use_autoconf, i)
  end; alias use_autoconf= set_autoconf # === use_autoconf=

  # ========================================================================= #
  # === set_pre_make_commands
  # ========================================================================= #
  def set_pre_make_commands(i)
    set(:pre_make_commands, i)
  end

  # ========================================================================= #
  # === set_headers
  # ========================================================================= #
  def set_headers(i)
    set(:headers, i)
  end

  # ========================================================================= #
  # === set_programs_dir
  #
  # This can set to another directory than '/Programs'.
  # ========================================================================= #
  def set_programs_dir(i = '/Programs/')
    @internal_hash[:programs_dir] = i
  end; alias programs_dir= set_programs_dir # === programs_dir=

  # ========================================================================= #
  # === set_blfs
  # ========================================================================= #
  def set_blfs(i)
    set(:blfs, i)
  end

  # ========================================================================= #
  # === set_configure_command
  # ========================================================================= #
  def set_configure_command(i)
    set(:configure_command, i)
  end; alias configure_command= set_configure_command # === configure_command=

  # ========================================================================= #
  # === set_md5sum
  # ========================================================================= #
  def set_md5sum(i)
    set(:md5sum, i)
  end

  # ========================================================================= #
  # === set_keep_extracted
  # ========================================================================= #
  def set_keep_extracted(i)
    set(:keep_extracted, i)
  end; alias keep_extracted= set_keep_extracted # === keep_extracted=
       
  # ========================================================================= #
  # === set_real_short_name
  #
  # This method should be used to set the value for the 'real_short_name'.
  #
  # The real short name entry entry may not include any '-' characters.
  #
  # Since Jan 2014 we also get rid of '_' characters - this is experimental
  # though and may be subject to change.
  # ========================================================================= #
  def set_real_short_name(
      i = short_name? # 1 tag
    )
    if i
      i = i.to_s.dup # Work on a copy to avoid side-effects.
      i.delete!('-') if i.include? '-'
      i.delete!('_') if i.include? '_'
    end
    set(:real_short_name, i)
  end; alias determine_real_short_name set_real_short_name # === determine_real_short_name

  # ========================================================================= #
  # === set_complex_program_version
  #
  # The purpose of this method is to allow you to overrule the
  # respective settings, at run-time.
  #
  # That way, you can change e. g. "glib-2.24" to "glib-2.25".
  # ========================================================================= #
  def set_complex_program_version(
      i,
      be_verbose = true
    )
    case be_verbose
    when :be_quiet, :be_silent
      be_verbose = false
    end
    # ======================================================================= #
    # Add a '=' to make it easier to know what we want to modify.
    # ======================================================================= #
    i.prepend '=' unless i.include? '='
    # ======================================================================= #
    # Next, we fetch a specific version. We assume this to be the last part
    # of the command given, which must include a '=' character.
    # ======================================================================= #
    new_version = i.split('=').last
    # ======================================================================= #
    # Chop off archive-extension since we don't need these.
    # ======================================================================= #
    new_version = remove_file_extension(new_version)
    if be_verbose
      opnn; e 'Setting a new version: '+simp(new_version)
    end
    set_program_version(new_version, :propagate_the_change) # The second argument will propagate the change, by cascading downwards.
    update_prefix :bypass_check
  end

  # ========================================================================= #
  # === set_short_name
  #
  # This method will set the short_name entry in the main yaml_dataset.
  #
  # The 'short_name' variable is allowed to include the '-' character,
  # and if the '-' character is part of the program name in question,
  # such as for a program like "gnome-commander", then it must be
  # retained in the method set_short_name() here.
  #
  # Numbers will be retained as well because they may be part of the
  # short_name, such as for the program called "dzen2".
  #
  # For the variant without '-' please see to the method
  # set_real_short_name() instead.
  #
  # Valid input to this method could be something like:
  #   "xfce4-dev-tools"
  # ========================================================================= #
  def set_short_name(
      i,             # 2 tag
      also_set_other_names = true
    )
    i = i.to_s.dup
    case also_set_other_names
    when :cascade_downwards,
         :cascade,
         :propagate_towards_real_short_name
      also_set_other_names = true
    when :do_not_cascade
      also_set_other_names = false
    end
    # ======================================================================= #
    # === Get rid of '/'
    # ======================================================================= #
    if i.include? '/'
      i = File.basename(i)
    end
    # ======================================================================= #
    # If the input includes .tar, check it further.
    # ======================================================================= #
    if i.include?('.tar')
      i = ::ProgramInformation.return_short_name(i)
    end
    set(:short_name, i)
    # ======================================================================= #
    # We can propagate only into (1) from this method, which is the
    # 'real_short_name' again.
    # ======================================================================= #
    if also_set_other_names
      set_real_short_name(i)      # By default, sync when setting short_name.
      set_program_compile_name(i) # Also sync it here.
    end
  end; alias set_real_program_name= set_short_name # === set_real_program_name=

  # ========================================================================= #
  # === set_apply_patch
  # ========================================================================= #
  def set_apply_patch(i = nil)
    set(:apply_patch, i)
  end

  # ========================================================================= #
  # === set_extract_to
  #
  # Consistently use this method when you wish to use the extract_to
  # variable.
  #
  # This variable must ALWAYS keep the location of where to extract a
  # package to lateron. 
  # ========================================================================= #
  def set_extract_to(i = TEMP_DIR)
    i = TEMP_DIR if i.nil? # default.
    i = i.dup # We don't want to keep a tainted reference.
    # ======================================================================= #
    # Ensure that we have a proper default value if the input was ''.
    # ======================================================================= #
    i = extract_to? if i.to_s.empty?
    # ======================================================================= #
    # Try to get an ENV variable here, if ENV is part of @extract_to.
    # ======================================================================= #
    if i.include? 'ENV'
      match = i.gsub(/ENV\['/,'').gsub(/'\]/,'') # contains the ENV variable
      i = ENV[match].dup
    end
    # ======================================================================= #
    # append / if the last character is not '/'
    # ======================================================================= #
    i << '/' unless i.end_with? '/'
    set(:extract_to, i)
  end; alias extract_to= set_extract_to # === extract_to=

  # ========================================================================= #
  # === set_env_variables
  # ========================================================================= #
  def set_env_variables(i)
    set(:set_env_variables, i)
  end; alias set_env_variable set_env_variables # === set_env_variable

  # ========================================================================= #
  # === set_short_description
  #
  # The short_description entry should not end with a newline.
  # ========================================================================= #
  def set_short_description(i = nil)
    i.chomp! if i # We do not want trailing newlines.
    set(:short_description, i)
  end; alias set_short_desc set_short_description # === set_short_desc

  # ========================================================================= #
  # === set_program_full_name
  #
  # We combine program_name? and archive_type? here.
  # ========================================================================= #
  def set_program_full_name(
      i = return_program_full_name
    )
    set(:program_full_name, i) 
  end; alias build_program_full_name  set_program_full_name # is equal to the default above method call.

  # ========================================================================= #
  # === assemble_program_full_name
  #
  # This method will build-up a valid "program_full_name" setting.
  #
  # In order to achieve this, it will make use of the method called
  # return_default_program_full_name, which itself is a combination
  # of program_name?+archive_type?.
  # ========================================================================= #
  def assemble_program_full_name(
      i = return_default_program_full_name
    )
    set_program_full_name(i)
  end; alias reassemble_program_full_name assemble_program_full_name # === reassemble_program_full_name
       alias determine_program_full_name  assemble_program_full_name # === determine_program_full_name

  # ========================================================================= #
  # === set_url2
  # ========================================================================= #
  def set_url2(i)
    set(:url2, i)
  end

  # ========================================================================= #
  # === chop_off_revision_number
  #
  # This method will check if a program version ends with a String such
  # as "-r8". If so then this part will be chopped off.
  #
  # Usually this trailing part denotes an internal revision rather than
  # an external program version.
  # ========================================================================= #
  def chop_off_revision_number
    _ = program_version?
    use_this_regex = /(-r\d{1,2})$/
    if _ =~ use_this_regex
      _.sub!(use_this_regex,'')
      set_program_version(_)
    end
  end

  # ========================================================================= #
  # === set_program_compile_name
  # ========================================================================= #
  def set_program_compile_name(i)
    set(:program_compile_name, i)
  end

  # ========================================================================= #
  # === set_url1
  # ========================================================================= #
  def set_url1(i)
    set(:url1, i)
  end

  # ========================================================================= #
  # === set_extra_information
  # ========================================================================= #
  def set_extra_information(i)
    set(:extra_information, i)
  end

  # ========================================================================= #
  # === decompose_program_name
  #
  # This method will set short_name and program_version, by making use of
  # the content stored in program_name? by default.
  # ========================================================================= #
  def decompose_program_name(i = program_name?)
    _ = ::ProgramInformation.return_short_name_and_program_version(i)
    set_short_name(_.short_name?, :propagate_towards_real_short_name)
    set_program_version(_.program_version?)
  end

  # ========================================================================= #
  # === set_prefix
  #
  # Use this method to set towards a specific prefix, such as '/usr/'.
  #
  # This is then the option that will be passed into scripts such as
  # 'configure' or cmake-based systems.
  # ========================================================================= #
  def set_prefix(i = '/usr/')
    case i
    when :appdir
      i = return_appdir_path
    end
    set(:prefix, i)
  end; alias prefix=        set_prefix # === prefix=
       alias prefix_to_use= set_prefix # === prefix_to_use=

  # ========================================================================= #
  # === use_autogen
  # ========================================================================= #
  def use_autogen(i = true)
    set(:use_autogen, i)
  end; alias use_autogen=    use_autogen # === use_autogen?
       alias set_use_autogen use_autogen # === set_use_autogen

  # ========================================================================= #
  # === set_homepage
  # ========================================================================= #
  def set_homepage(i = nil)
    set(:homepage, i)
  end

  # ========================================================================= #
  # === set_configure_options
  # ========================================================================= #
  def set_configure_options(i)
    # ======================================================================= #
    # Since as of September 2017, the input is no longer allowed to have
    # any newlines. This is explicitely mentioned here in the event that
    # we may have to rever this decision one day.
    # ======================================================================= #
    if i.include? N
      i.delete!(N)
      i.squeeze!(' ') # And also get rid of '  ' variants.
    end
    set(:configure_options, i)
  end; alias configure_options= set_configure_options # === configure_options=

  # ========================================================================= #
  # === do_use_this_program_version
  #
  # Use this method specifically to set (and use) another program version.
  #
  # The difference towards set_program_version is that we will also
  # re-assemble the default program path.
  # ========================================================================= #
  def do_use_this_program_version(i = '1.52.1')
    set_program_version(i)
    reassemble_program_path
  end; alias use_this_program_version do_use_this_program_version # === use_this_program_version
       alias do_use_this_version=     do_use_this_program_version # === do_use_this_version=

  # ========================================================================= #
  # === set_last_update
  # ========================================================================= #
  def set_last_update(i)
    set(:last_update, i)
  end

  # ========================================================================= #
  # === set_archive_size
  #
  # The archive size, that is, how large the archive is - in bytes.
  #
  # Will default to 0 on startup or when no argument is passed to it.
  # ========================================================================= #
  def set_archive_size(i = 0)
    set(:archive_size, i)
  end

  # ========================================================================= #
  # === assume_default_archive_type
  # ========================================================================= #
  def assume_default_archive_type
    set_archive_type(DEFAULT_ARCHIVE_TYPE)
  end

  # ========================================================================= #
  # === set_archive_type                                 (archive tag, 4 tag)
  #
  # This method will set the 'archive_type' of a given program at hand.
  #
  # This is entry number (4), such as '.tar.xz' in a program like
  # 'gnome-mahjonng-5.08.tar.xz'.
  #
  # Typically, archives will be kept in the .tar.xz format, or rather
  # the content of the constant DEFAULT_ARCHIVE_TYPE. We need this
  # part when we wish to assemble the program_full_name setting
  # lateron, at entry number (7).
  # ========================================================================= #
  def set_archive_type(i = DEFAULT_ARCHIVE_TYPE)
    case i
    when nil, :default, 'default'
      i = DEFAULT_ARCHIVE_TYPE # Safeguard.
    end
    i = i.to_s.dup.downcase # Keep it downcased as well, and in String format.
    # ======================================================================= #
    # === Get rid of '/'
    # ======================================================================= #
    if i.include? '/'
      i = File.basename(i)
    end
    # ======================================================================= #
    # Do some sanitizing next.
    # ======================================================================= #
    i.sub!(/\.source$/,'') if i.include? '.source'
    # ======================================================================= #
    # If the input includes a '-' token, then something can not be right.
    # ======================================================================= #
    if i.include? '-'
      i = i[(i.rindex('-')+1)..-1]
    # ======================================================================= #
    # Same for input containing '_'.
    # ======================================================================= #
    elsif i.include? '_'
      i = i[(i.rindex('_')+1)..-1]
    end
    if i == 'guess'
      i = File.basename(url?)
    end
    i = return_archive_type(i) # This method must have registered all archive-types.
    set(:archive_type, i)
  end; alias set_archive        set_archive_type # === set_archive
       alias set_archive_format set_archive_type # === set_archive_format
       alias set_this_archive   set_archive_type # === set_this_archive

  # ========================================================================= #
  # === set_program_name                                     (entry number 6)
  #
  # The program name of a program might be something such as this:
  #
  #   gnome-mahjonng-5.08
  #   file-5.05
  #
  # So in other words, this is entry number (6) and it will combine
  # the two entries (2) and (3): 
  #
  #   short_name?+'-'+program_version?
  #
  # A '-' is usually required as separator for a proper "program_name"
  # entry, about at the least in 99.5% of the programs out there in
  # the wild. They usually include a '-' as part of the program name.
  # Note that there are some weird exceptions such as "boost_1_59_0".
  #
  # Consistently use this and ONLY this method when modifying the
  # 'program_name' entry in the main dataset.
  #
  # One more question should be answered - why do we use
  # real_short_name? rather than short_name?
  #
  # We need to use real_short_name? because some other settings may
  # require input without any '-' as part of their name.
  #
  # Some other programs require to have a '_' as part of the program name.
  #
  # This can lead to problems such as for those programs that have a _ as
  # part of their name. A good example is the name "e_dbus", which includes
  # includes a '_' token as part of its name.
  #
  # In Jan 2014, I realized that eliminating '_' is too much of a hassle,
  # so I disabled it again. I will retain the explanation above though,
  # in the event that future programmers - or future me - may encounter
  # similar problems.
  # ========================================================================= #
  def set_program_name(i = return_program_name)
    # ======================================================================= #
    # A program name never has a '/'.
    # ======================================================================= #
    if i.include? '/'
      i = File.basename(i)
    end
    i = remove_file_extension(i) if i.include? '.' # <- Added this as of Nov 2013.
    case i
    when 'URL_REPLACE_UNDERSCORE'
      i = url?.to_s.dup
    when '', nil, :assemble, :guess # Then we assemble it for these two conditions.
      assemble_program_name # In these cases, we will try to guess the name.
    end
    i = i.to_s.dup
    # if i.include?('_') and !i.include?('-')
    #   i.tr!('_','-')
    # end # ^^^ this line would break artemis.
    set(:program_name, i)  
  end; alias program_name= set_program_name # === program_name=

  # ========================================================================= #
  # === return_program_name
  #
  # Note that the connective token in use is usually '-', but there are
  # programs such as 'artemis_v16.0.11.jar' that have no '-' at all
  # whatsoever. To also honour this, we have to use this connective
  # token - it will be re-defined towards '_' specifically for programs
  # that have no '-' token.
  # ========================================================================= #
  def return_program_name(
      use_this_connective_token = use_this_connective_token?
    )
    short_name?+use_this_connective_token+program_version?
  end

  # ========================================================================= #
  # === set_use_this_connective_token
  # ========================================================================= #
  def set_use_this_connective_token(i = '-')
    @internal_hash[:use_this_connective_token] = i
  end; alias use_default_connective_token_again set_use_this_connective_token # === use_default_connective_token_again

  # ========================================================================= #
  # === assemble_default_program_name
  #
  # This method will assemble the program_name (6) variable, which
  # is derived directly from:
  #
  #  short_name? and '-' and program_version?
  #
  # For example it could look like this:
  #   "gnome-mahjongg-5.08"
  # ========================================================================= #
  def assemble_default_program_name
    set_program_name(return_program_name)
  end; alias update_program_full_name                assemble_default_program_name # === update_program_full_name
       alias assemble_program_name                   assemble_default_program_name # === assemble_program_name
       alias update_program_name                     assemble_default_program_name # === update_program_name
       alias attach_program_name_and_program_version assemble_default_program_name # === attach_program_name_and_program_version

  # ========================================================================= #
  # === set_program_version                                     (version tag)
  #
  # This method will set the program_version.
  #
  # Use this method whenever you wish to modify the version of
  # the program, which is entry number (3), like:
  #
  #   "5.08"
  # ========================================================================= #  
  def set_program_version(i)
    i = i.to_s.dup # Work on a copy.
    i.sub!(/-source$/,'') if i.include? '-source'
    i.sub!(/\.src/,'')    if i.include? '.src'
    i.sub!(/-Source/,'')  if i.include? '-Source'
    i = chop_off_archive(i)
    i = remove_file_suffix(i) # Added this as of Jan 2014.
    if i.include? '#' # This is an illegal character as of Apr 2014.
      i = i[0, i.index('#')]
    end
    set(:program_version, i)
  end; alias set_program_version= set_program_version # === set_program_version=
       alias set_version          set_program_version # === set_version
       alias use_this_version=    set_program_version # === use_this_version=

  # ========================================================================= #
  # === set_licence
  # ========================================================================= #
  def set_licence(i)
    set(:licence, i)
  end; alias set_license set_licence # === set_license

  # ========================================================================= #
  # === set_description
  #
  # Note that we will always STORE the full description - it is only the
  # actual display that will be truncated.
  # ========================================================================= #
  def set_description(i)
    set(:description, i)
  end

  # ========================================================================= #
  # === set_raw_description
  # ========================================================================= #
  def set_raw_description(i)
    set(:raw_description, i.to_s.dup) # Keep a reference-copy to the original description here.
  end

  # ========================================================================= #
  # === set_default_raw_description
  # ========================================================================= #
  def set_default_raw_description
    set_raw_description(description?(:retain_newlines).to_s) # We must retain the original newlines here.
  end

  # ========================================================================= #
  # === set_program_path                                              (8 tag)
  #
  # The program_path is composed of these two elements:
  #
  #    - base_dir?
  #    - program_full_name?
  #
  # It would look like this:
  #
  #   '/Users/x/SRC/gnomemahjongg/gnome-mahjonng-5.08.tar.xz'
  #
  # In other words, it will add (5) + (7).
  #
  # It is the most important entry, containing everything - which is the
  # major goal for the class here.
  #
  # The full program_path will look such as:
  #
  #   '/Users/x/SRC/file/file-5.08.tar.xz'
  #   '/Users/x/SRC/gnomemahjongg/gnome-mahjonng-5.08.tar.xz'
  #
  # The two main components are thus:
  #
  #   (5) base_dir,          such as: "/Users/x/SRC/file/"
  #   (7) program_full_name, such as: "file-5.08.tar.xz"
  # ========================================================================= #
  def set_program_path(i)
    set(:program_path, i)
  end

  # ========================================================================= #
  # === assemble_program_path
  #
  # This method will always ultimate attempt to set the program path, if
  # possible.
  # ========================================================================= #
  def assemble_program_path(
      i = return_default_program_path
    )
    assemble_default_program_name # Assign to something such as: "gnome-mahjongg-0.1.3"
    set_program_path(i)
  end; alias reassemble_program_path assemble_program_path # === reassemble_program_path
       alias assemble_full_path      assemble_program_path # === assemble_full_path
       alias update_program_path     assemble_program_path # === update_program_path

  # ========================================================================= #
  # === symlink_glib_schema
  # ========================================================================= #
  def set_symlink_glib_schema(i)
    set(:symlink_glib_schema, i)
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    # ======================================================================= #
    # Keep track of the commandline arguments in the main Hash of this class.
    # ======================================================================= #
    @internal_hash[:commandline_arguments] = i
    if i.is_a? Array
      i = i.join(' ').strip
    end 
    i = i.to_s.dup
    i.delete!('-') if i.include? '-'
    # ======================================================================= #
    # And then, we will seek for this program.
    # ======================================================================= #
    set_search_for_this_program(i)
  end

  # ========================================================================= #
  # === set_search_for_this_program
  #
  # Set the main program to work with. In other words, we will seek this
  # program in the cookbooks "database".
  #
  # This should always be at least a String.
  #
  # Keep in mind that the input to this method may also be in the form
  # of a String such as "php.yml". In that case, we have to get rid
  # of the trailing '.yml' part.
  #
  # Additionally, do note that the input could be a binary name or a
  # library name. In these two cases, we may overrule the input provided
  # but this can be configured via some settings.
  #
  # Since as of Sep 2012, aliases can overrule this setting. This however
  # may lead to problems, such as "make" suddenly becoming "makeinfo"
  # rather than being kept as make.
  # ========================================================================= #
  def set_search_for_this_program(
      i = DEFAULT_ENTRY
    )
    i = i.keys.first if i.is_a? Hash
    i.flatten! if i.is_a? Array # Keep it flattened.
    i = DEFAULT_ENTRY if i.nil?
    i = i.first if i.is_a? Array
    i = i.to_s.dup # Work on a reference copy.
    # ======================================================================= #
    # Pass the input into the menu() here, but only if we will use the
    # menu interface.
    # ======================================================================= #
    menu(i) if @use_menu_interface
    i.delete!('-_') # Neither '-' nor '_' are allowed as input.
    if i.include? '.'
      i.sub!(/#{File.extname(i)}/,'')
    end
    # ======================================================================= #
    # We will also chop off trailing '/' characters. A trailing '/' is
    # assumed to have been a typo. Nobody will call a file or input
    # program something such as 'libgweather/'.
    # ======================================================================= #
    i.chop! if i.end_with? '/'
    # ======================================================================= #
    # === Next check for entries that lack a leading 'lib' entry
    # ======================================================================= #
    unless does_include?(i, :do_not_check_for_aliases)
      if does_include?('lib'+i, :do_not_check_for_aliases)
        opnn; e 'The given input '+sfancy(i)+' is not included '\
                'but '+sfancy('lib'+i)+'. is.'
        opnn; e 'Thus using this as the new input next.'
        i.prepend('lib')
      end
    end
    all_binaries  = all_binaries?
    all_libraries = all_libraries?
    # ======================================================================= #
    # === Handle possible binary names next
    #
    # Here we will check for binary names of programs, but only if the input
    # has not yet been found as regular name of a registered program at
    # hand. As this is not always needed anyway, we will use various checks
    # to make sure that we really want to find a binary name.
    #
    # In order to test this, try:
    #   cookbooks2 xmlif
    #   ry xmlif
    # ======================================================================= #
    if !does_include?(i) and check_for_binary_names? and
       all_binaries.has_key?(i) and (i != all_binaries[i])
      new_name = all_binaries[i]
      report_binary_or_library_program_name(i, new_name, :binary)
      i = new_name
    # ======================================================================= #
    # === Check for library names
    #
    # Similar reasoning applies to this entry point, as to the one above
    # for binary names.
    # ======================================================================= #
    elsif !does_include?(i) and check_for_library_names? and
       all_libraries.has_key?(i) and (i != all_libraries[i])
      new_name = all_libraries[i]
      report_binary_or_library_program_name(i, new_name, :library)
      i = new_name
    end
    # ======================================================================= #
    # === Check for Abbreviations if the key was not found
    # ======================================================================= #
    unless does_include?(i, :do_not_check_for_aliases)
      unless i.empty?
        # =================================================================== #
        # Ok, we did not find anything, so check for abbreviations.
        # =================================================================== #
        available_programs = available_programs?
        require 'abbrev' unless Object.const_defined? :Abbrev
        abbrevs = Abbrev.abbrev(available_programs)
        if abbrevs.has_key? i
          # ================================================================= #
          # Ok, this means that our key is included.
          # ================================================================= #
          i = abbrevs.fetch(i)
        end
      end
    end
    # ======================================================================= #
    # Next, set it but not if it is empty.
    # ======================================================================= #
    unless i.empty?
      @internal_hash[:search_for_this_program] = i
    end
  end; alias seek_this_program     set_search_for_this_program # === seek_this_program
       alias seek                  set_search_for_this_program # === seek
       alias set_seek_this_program set_search_for_this_program # === set_seek_this_program
       alias set_search_term       set_search_for_this_program # === set_search_term

  # ========================================================================= #
  # === set_make_use_of_aliases_kept_in_the_file_cookbook_aliases
  # ========================================================================= #
  def set_make_use_of_aliases_kept_in_the_file_cookbook_aliases(i)
    case i
    when :nope, :user_disabled_this_setting_via_the_commandline
      i = false
    end
    @internal_hash[:make_use_of_aliases_kept_in_the_file_cookbook_aliases] = i
  end

  # ========================================================================= #
  # === set_required_deps_on
  #
  # Set the required dependencies for a program through this method.
  #
  # Note that this method will not do any sanitizing - you have to sanitize
  # the input on your own prior to calling this method.
  # ========================================================================= #
  def set_required_deps_on(i)
    set(:required_deps_on, i)
  end

  # ========================================================================= #
  # === set_has_to_be_compiled
  # ========================================================================= #
  def set_has_to_be_compiled(i = true)
    set(:has_to_be_compiled, i)
  end

  # ========================================================================= #
  # === set_use_this_make_command
  # ========================================================================= #
  def set_use_this_make_command(i)
    set(:use_this_make_command, i)
  end; alias use_this_make_command= set_use_this_make_command # === use_this_make_command=

  # ========================================================================= #
  # === set_use_this_make_install_command
  # ========================================================================= #
  def set_use_this_make_install_command(i)
    set(:use_this_make_install_command, i)
  end; alias use_this_make_install_command= set_use_this_make_install_command # === use_this_make_install_command=

  # ========================================================================= #
  # === set_make_options
  # ========================================================================= #
  def set_make_options(i)
    set(:make_options, i)
  end

  # ========================================================================= #
  # === set_base_dir                      (base_dir tag, base dir tag, 5 tag)
  #
  # This setter will set the base_directory setting, which is entry
  # number (5).
  #
  # An example for a proper base_directory setting would be a String such
  # as this one here:
  #
  #   "/Users/x/SRC/file/"
  #
  # If the input is :guess_base_dir then this means that we will try
  # to guess the base directory.
  #
  # Also note that real_short_name? should be set before calling this
  # method, in some situations.
  # ========================================================================= #
  def set_base_dir(
      i = source_dir?+
          real_short_name?.downcase.delete('.')+ # Note that the base_dir part is not allowed to have any '.' characters.
          '/'
    )
    case i
    when :guess_base_dir, :guess
      i = guess_base_dir
    end
    # ======================================================================= #
    # We don't want any '-' in the base_dir name.
    # ======================================================================= #
    i.delete!('-') if i.include? '-'
    # ======================================================================= #
    # === Delete _ from the name of the directory
    #
    # Added the following code as of January 2014. While programs can have
    # names such as 'e_dbus', the directory name itself is not allowed
    # to contain any '_' token. It looks visually odd otherwise.
    # ======================================================================= #
    i.delete!('_') if i.include? '_'
    # ======================================================================= #
    # === Handle + tokens
    #
    # Some directory names may include a '+' character, such as is the
    # case with gtk+. There are several ways how to deal with this, but
    # if we have a certain configuration option set, then we will
    # replace '+' with long name, which is 'plus'. I personally no 
    # longer use this - my directory names will also include a '+'
    # character as a consequence.
    # ======================================================================= #
    if i.include? '+'
      i.gsub!(/\+/, 'plus') if replace_plus_with_long_name?
    end
    i = rds(i) # Remove double slashes
    # ======================================================================= #
    # Assume that a path starting with ./ is never correct.
    # ======================================================================= #
    if i.start_with? './'
      i[0,2] = source_dir?
    end
    # ======================================================================= #
    # === Append /
    #
    # It is the correct way to ensure that the last char is a /. The
    # reason is: directories contain a '/' as part of their name.
    # ======================================================================= #
    i << '/' unless i.end_with? '/'
    set(:base_dir, i)
  end; alias determine_base_directory  set_base_dir # === determine_base_directory
       alias determine_base_dir        set_base_dir # === determine_base_dir
       alias set_base_dir=             set_base_dir # === set_base_dir=
       alias base_dir=                 set_base_dir # === base_dir=
       alias try_to_guess_the_base_dir set_base_dir # === try_to_guess_the_base_dir

  # ========================================================================= #
  # === set_skip_configure
  # ========================================================================= #
  def set_skip_configure(i)
    set(:skip_configure, i)
  end

  # ========================================================================= #
  # === set_parameters_to_make
  # ========================================================================= #
  def set_parameters_to_make(i)
    set(:parameters_to_make, i)
  end

end; end