#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# Some colours are hardcoded here.
# =========================================================================== #
# require 'cookbooks/colours/colours.rb'
# =========================================================================== #
require 'colours'
require 'cookbooks/constants/configuration_constants.rb'
require 'cookbooks/constants/misc.rb'
require 'cookbooks/module_methods/is_on_roebe.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks::KONSOLE
  # ========================================================================= #
  KONSOLE = ::Colours::Konsole # Easier alias to the konsole-colours.

  # ========================================================================= #
  # === @use_colours
  #
  # This variable determines whether to use colours for the Cookbooks
  # project. A configuration option sets this variable.
  # ========================================================================= #
  @use_colours = YAML.load_file(FILE_USE_COLOURS)

  # ========================================================================= #
  # === Cookbooks.use_colours?
  # ========================================================================= #
  def self.use_colours?
    @use_colours
  end

  # ========================================================================= #
  # === Cookbooks.disable_colours
  #
  # This method can be used to globally disable the colours.
  # ========================================================================= #
  def self.disable_colours
    @use_colours = false
  end; self.instance_eval { alias do_not_use_colours disable_colours } # === Cookbooks.do_not_use_colours

  # ========================================================================= #
  # === Cookbooks.simp
  # ========================================================================= #
  def self.simp(i)
    return ::Colours.simp(i) if Cookbooks.use_colours?
    i
  end

  # ========================================================================= #
  # === Cookbooks.sfile
  # ========================================================================= #
  def self.sfile(i)
    return ::Colours.sfile(i) if Cookbooks.use_colours?
    i
  end

  # ========================================================================= #
  # === Cookbooks.sfancy
  # ========================================================================= #
  def self.sfancy(i)
    return ::Colours.sfancy(i) if Cookbooks.use_colours?
    i
  end

  # ========================================================================= #
  # === Cookbooks.sdir
  # ========================================================================= #
  def self.sdir(i)
    return ::Colours.sdir(i) if Cookbooks.use_colours?
    i
  end

  # ========================================================================= #
  # === Cookbooks.swarn
  # ========================================================================= #
  def self.swarn(i)
    return ::Colours.swarn(i) if Cookbooks.use_colours?
    i
  end

  # ========================================================================= #
  # === Cookbooks.violet
  # ========================================================================= #
  def self.violet(i = '')
    return KONSOLE.violet(i) if Cookbooks.use_colours?
    i
  end

  # ========================================================================= #
  # === bold_green
  # ========================================================================= #
  def bold_green(i)
    if Cookbooks.use_colours?
      "\e[1;32m"+i+"\e[0;37m"
    else
      i
    end
  end

  # ========================================================================= #
  # === bold_blue
  # ========================================================================= #
  def bold_blue(i)
    if Cookbooks.use_colours?
      "\e[1;34m"+i+"\e[0;37m"
    else
      i
    end
  end

  # ========================================================================= #
  # === orange
  # ========================================================================= #
  def self.orange(i)
    return KONSOLE.orange(i) if Cookbooks.use_colours?
    i
  end

  # ========================================================================= #
  # === crimson
  # ========================================================================= #
  def self.crimson(i)
    return KONSOLE.crimson(i) if Cookbooks.use_colours?
    i
  end

  # ========================================================================= #
  # === turquoise
  # ========================================================================= #
  def self.turquoise(i)
    return KONSOLE.turquoise(i) if Cookbooks.use_colours?
    i
  end

  # ========================================================================= #
  # === Cookbooks.permanently_disable_colours
  #
  # This method can be used to permanently disable the colours for
  # the Cookbooks project. This essentially just saves into the
  # file 'use_colours.yml'.
  #
  # To invoke this from the commandline, do:
  #   cookbooks --permanently-disable-colours
  # ========================================================================= #
  def self.permanently_disable_colours
    into = FILE_USE_COLOURS
    what = YAML.dump(false)
    opn; e 'Permanently disabling the colours for the '\
           'Cookbooks project.'
    opn; e "(file: #{sfile(into)})"
    write_what_into(what, into)
    if is_on_roebe?
      into = "#{RUBY_SRC_DIR_AT_HOME}cookbooks/lib/cookbooks/yaml/configuration/"\
             "#{File.basename(FILE_USE_COLOURS)}"
      opn; e 'Permanently disabling the colours for the '\
             'Cookbooks project.'
      opn; e "(file: #{sfile(into)})"
      write_what_into(what, into)
    end
  end

end