#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::SimpleConfigurationLoader
#
# This class will load the various .yml files from the config. This
# is to be done when a user does not have the configuration project
# available. 
#
# Usage example:
#   Cookbooks::SimpleConfigurationLoader.new
# =========================================================================== #
# require 'cookbooks/configuration/simple_configuration_loader.rb'
# =========================================================================== #
require 'cookbooks/constants/configuration_constants.rb'
require 'cookbooks/requires/require_yaml.rb'

module Cookbooks

class SimpleConfigurationLoader # === Cookbooks::SimpleConfigurationLoader.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    @array_available_methods = []
  end

  # ========================================================================= #
  # === obtain_all_available_yaml_files
  #
  # This method will fetch every available .yml file that is part of
  # the "configuration system" for the Cookbooks.
  # ========================================================================= #
  def obtain_all_available_yaml_files(
      from_this_dir = CONFIGURATION_DIRECTORY
    )
    @yaml_files = Dir["#{from_this_dir}*.yml"]
  end

  # ========================================================================= #
  # === iterate_over_the_yaml_files
  # ========================================================================= #
  def iterate_over_the_yaml_files
    _ = @yaml_files
    unless _.empty?
      _.each {|this_yaml_file|
        name_of_the_method_that_is_to_be_defined = File.basename(
          this_yaml_file.sub(/\.yml$/,'')
        ).to_sym
        # =================================================================== #
        # Register which automatically defined methods will be available.
        # These can then be used e. g. .use_colours()
        # =================================================================== #
        @array_available_methods << name_of_the_method_that_is_to_be_defined
        data = YAML.load_file(this_yaml_file)
        self.class.class_eval {
          define_method(name_of_the_method_that_is_to_be_defined) {
            data
          }
        }
      }
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    obtain_all_available_yaml_files
    iterate_over_the_yaml_files
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  pp Cookbooks::SimpleConfigurationLoader.new(ARGV)
end # simple_configuration_loader