#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks
#
# This file will define some constants for the cookbooks project.
#
# It will also properly initialize the Configuration dataset.
# =========================================================================== #
# require 'cookbooks/constants/constants.rb'
# =========================================================================== #
require 'cookbooks/project_settings/project_yaml_directory.rb'
require 'cookbooks/constants/misc.rb'
require 'cookbooks/constants/namespace.rb'

module Cookbooks

  # ========================================================================= #
  # === N
  # ========================================================================= #
  N = "\n"

  # ========================================================================= #
  # === DEFAULT_PROGRAM_ENTRY
  # ========================================================================= #
  DEFAULT_PROGRAM_ENTRY = 'htop'
          DEFAULT_ENTRY = DEFAULT_PROGRAM_ENTRY # an "alias" to the above ^^^

  # ========================================================================== #
  # === MAKE_COMMAND_TO_USE
  #
  # Which make command to use.
  # ========================================================================== #
  MAKE_COMMAND_TO_USE = 'make'

  # ========================================================================= #
  # === DEPOT_DIR
  # ========================================================================= #
  DEPOT_DIR = '/Depot/'

  # ========================================================================== #
  # === COMPILE_EVERYTHING_INTO_STANDALONE_DIRECTORY
  #
  # Whether to compile everything into a standalone directory or not.
  # ========================================================================== #
  COMPILE_EVERYTHING_INTO_STANDALONE_DIRECTORY = false

  # ========================================================================= #
  # === DEFAULT_COLOUR_TO_USE
  #
  # Specify what is our main colour to use for the Cookbooks project.
  # ========================================================================= #
  DEFAULT_COLOUR_TO_USE = "\e[0;32m" # This is GREEN, which is defined in module Colours.

  # ========================================================================= #
  # === ROEBE_PATH_TO_COOKBOOKS
  #
  # Path to my cookbooks. Won't be too relevant for other people.
  # ========================================================================= #
  ROEBE_PATH_TO_COOKBOOKS =
    RUBY_SRC_DIR_AT_HOME+'cookbooks/lib/cookbooks/'

  # ========================================================================= #
  # === SYSBIN_DIRECTORY
  # ========================================================================= #
  SYSBIN_DIRECTORY = '/System/Executables/'

  # ========================================================================== #
  # === Cookbooks::FILE_COOKBOOK_CHAINED_PROGRAMS
  #
  # Easier constant to keep track of the chained cookbook programs.
  #
  # This will then yield to a path such as this here:
  #
  #   "/Programs/Ruby/2.3.1/lib/ruby/site_ruby/2.3.0/cookbooks/yaml/cookbook_chained_programs.yml"
  # ========================================================================== #
  FILE_COOKBOOK_CHAINED_PROGRAMS =
    COOKBOOKS_YAML_DIRECTORY+'cookbook_chained_programs.yml'

  # ========================================================================== #
  # === Cookbooks::FILE_USE_MD5SUM
  # ========================================================================== #
  FILE_USE_MD5SUM = COOKBOOKS_YAML_DIRECTORY+'configuration/use_md5sum.yml'

  # ========================================================================== #
  # === Cookbooks::FILE_ALLOWED_TAGS
  # ========================================================================== #
  FILE_ALLOWED_TAGS = COOKBOOKS_YAML_DIRECTORY+'allowed_tags.yml'

  # ========================================================================== #
  # === Cookbooks::FILE_PROGRAMS_VERSION
  #
  # This file, when existing, holds a shorter name: version dataset for
  # all programs.
  # ========================================================================== #
  FILE_PROGRAMS_VERSION = COOKBOOKS_YAML_DIRECTORY+'programs_version.yml'

  # ========================================================================== #
  # === Cookbooks.file_cookbook_chained_programs
  # ========================================================================== #
  def self.file_cookbook_chained_programs
    FILE_COOKBOOK_CHAINED_PROGRAMS
  end

  # ========================================================================== #
  # === home_dir?
  # ========================================================================== #
  def home_dir?
    Cookbooks.home_dir?
  end

  # ========================================================================== #
  # === Cookbooks::FILE_SHORT_TUTORIAL
  # ========================================================================== #
  FILE_SHORT_TUTORIAL =
    Cookbooks::PROJECT_BASE_DIRECTORY+'tutorial/short_tutorial.md' 

  # ========================================================================== #
  # === Cookbooks.file_short_tutorial?
  # ========================================================================== # 
  def self.file_short_tutorial?
    FILE_SHORT_TUTORIAL
  end

  # ========================================================================== #
  # === FILE_REGISTERED_HEADERS
  # ========================================================================== #
  FILE_REGISTERED_HEADERS =
    Cookbooks.yaml_directory?+'registered_headers.yml'

  # ========================================================================== #
  # === CHECK_FOR_BINARY_NAMES
  # ========================================================================== #
  CHECK_FOR_BINARY_NAMES  = true

  # ========================================================================== #
  # === CHECK_FOR_LIBRARY_NAMES
  # ========================================================================== #
  CHECK_FOR_LIBRARY_NAMES = true

  # ========================================================================== #
  # === SHOW_EXTRA_INFORMATION
  # ========================================================================== #
  SHOW_EXTRA_INFORMATION = true

end