# =========================================================================== #
# require 'cookbooks/constants/predefined_installation_instructions.rb'
# =========================================================================== #
require 'cookbooks/project_settings/project_yaml_directory.rb'
require 'cookbooks/requires/require_yaml.rb'

module Cookbooks

  # ========================================================================= #
  # === HASH_INSTALLATION_INSTRUCTIONS
  #
  # This Hash gives us an easier handle to sanitize our data.
  #
  # It has some constant-shortcuts that are convenient for use lateron.
  #
  # For instance, RINSTALL means "ruby install", PINSTALL means
  # "python install".
  # ========================================================================= #
  HASH_INSTALLATION_INSTRUCTIONS = {}

  # ========================================================================= #
  # === FILE_PREDEFINED_INSTALLATION_INSTRUCTIONS
  #
  # Define where the yaml file is kept.
  # ========================================================================= #
  FILE_PREDEFINED_INSTALLATION_INSTRUCTIONS = 
    ::Cookbooks.project_yaml_directory?+'predefined_installation_instructions.yml'

  # ========================================================================= #
  # === Cookbooks.file_predefined_installation_instructions
  #
  # Method to return where the file location is.
  # ========================================================================= #
  def self.file_predefined_installation_instructions
    FILE_PREDEFINED_INSTALLATION_INSTRUCTIONS
  end

  # ========================================================================= #
  # === PREDEFINED_INSTALLATION_INSTRUCTIONS
  #
  # Load up the dataset here.
  # ========================================================================= #
  if File.exist? FILE_PREDEFINED_INSTALLATION_INSTRUCTIONS
    PREDEFINED_INSTALLATION_INSTRUCTIONS = YAML.load_file(
      FILE_PREDEFINED_INSTALLATION_INSTRUCTIONS
    )
    PII = PREDEFINED_INSTALLATION_INSTRUCTIONS # Easier alias to it.
    PII.each_pair { |key, value|
      HASH_INSTALLATION_INSTRUCTIONS[key] = value
    }
  else
    PII = {} # Define an empty Hash if the file does not exist.
  end

  # ========================================================================= #
  # === Cookbooks.pii?
  # ========================================================================= #
  def self.pii?
    PII
  end

end

if __FILE__ == $PROGRAM_NAME
  require 'pp'
  pp Cookbooks::HASH_INSTALLATION_INSTRUCTIONS
end