#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/constants/registered_cookbook_entries.rb'
# =========================================================================== #
require 'cookbooks/project_settings/project_yaml_directory.rb'
require 'cookbooks/requires/require_yaml.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks::FILE_REGISTERED_COOKBOOK_ENTRIES
  # ========================================================================= #
  FILE_REGISTERED_COOKBOOK_ENTRIES =
    "#{::Cookbooks.yaml_directory?}registered_cookbook_entries.yml"  

  # ========================================================================= #
  # === Cookbooks.file_registered_cookbook_entries
  #
  # Tell use where the file registered_cookbook_entries.yml is stored.
  #
  # It is a part of the SanitizeCookbookDataset project.
  # ========================================================================= #
  def self.file_registered_cookbook_entries
    FILE_REGISTERED_COOKBOOK_ENTRIES
  end

  # ========================================================================= #
  # === Cookbooks::REGISTERED_COOKBOOK_ENTRIES
  #
  # The following constant contains all registered entries in the cookbook.
  #
  # These can be found specified in the file registered_cookbook_entries.yml.
  #
  # It is an Array.
  #
  # The file itself resides at:
  #
  #   $RSRC/cookbooks/lib/cookbooks/yaml/registered_cookbook_entries.yml
  #
  # ========================================================================= #
  if File.exist? FILE_REGISTERED_COOKBOOK_ENTRIES
    REGISTERED_COOKBOOK_ENTRIES =
      YAML.load_file(FILE_REGISTERED_COOKBOOK_ENTRIES)
  else
    REGISTERED_COOKBOOK_ENTRIES = []
  end

  # ========================================================================= #
  # === Cookbooks.registered_cookbook_entries?
  #
  # Simply return all registered cookbook entries.
  # ========================================================================= #
  def self.registered_cookbook_entries?
    REGISTERED_COOKBOOK_ENTRIES
  end; self.instance_eval { alias registered_cookbook_entries registered_cookbook_entries? } # === Cookbooks.registered_cookbook_entries?

  # ========================================================================= #
  # === registered_cookbook_entries?
  #
  # Just a pointer to the module-method with the same name.
  # ========================================================================= #
  def registered_cookbook_entries?
    Cookbooks.registered_cookbook_entries?
  end

end

if __FILE__ == $PROGRAM_NAME
  pp Cookbooks.registered_cookbook_entries?
end