#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::GobolinuxNamingConvention
#
# This class can be used as a drop-in replacement for the GoboLinux
# NamingConvention script.
#
# That is, it will convert a given input name, into a GoboLinux-conforming
# name.
#
# The following conventions have been recorded so far:
#
#   - "lib" will become "Lib"
#   - "bin" will become "Bin"
#   - "info" will become "INFO"
#   - "mate-desktop" will become "Mate-Desktop"
#
# Additionally, all three character parts, such as "abc-",
# will also be upcased.
#
# Usage example:
#   Cookbooks::GobolinuxNamingConvention.new
# =========================================================================== #
# require 'cookbooks/gobolinux/gobolinux_naming_convention.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'

module Cookbooks

class GobolinuxNamingConvention < Base # === GobolinuxNamingConvention.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = @input.downcase.dup
    if _.include? 'lib'
      _.gsub!(/lib/,'Lib')
    end
    if _.include? 'bin'
      _.gsub!(/bin/,'Bin')
    end
    if _.size < 4
      _.upcase!
    end
    # ======================================================================= #
    # === Check for stuff such as "libdrm", which will become "LibDRM".
    # ======================================================================= #
    if _.start_with? 'Lib' and _.size == 6
      _[-3,3] = _[-3,3].upcase
    end
    if _.include? 'info'
      _.sub!(/info/,'INFO')
    end
    if _.include? '-'
      splitted = _.split('-')
      splitted.map {|entry|
        entry[0,1] = entry[0,1].upcase
        entry
      }
      _ = splitted.join('-')
    end
    # ======================================================================= #
    # The first character will ALWAYS be upcased, no matter what.
    # ======================================================================= #
    _[0,1] = _[0,1].upcase
    set_translated_name(_)
  end

  # ========================================================================= #
  # === set_translated_name
  # ========================================================================= #
  def set_translated_name(i)
    @translated_name = i
  end

  # ========================================================================= #
  # === translated_name?
  # ========================================================================= #
  def translated_name?
    @translated_name
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end

# =========================================================================== #
# === Cookbooks.gobolinux_naming_convention
# =========================================================================== #
def self.gobolinux_naming_convention(i)
  i = i.first if i.is_a? Array
  GobolinuxNamingConvention.new(i).translated_name?
end; self.instance_eval { alias naming_convention     gobolinux_naming_convention } # === Gobolinux.naming_convention
     self.instance_eval { alias return_gobolinux_name gobolinux_naming_convention } # === Gobolinux.return_gobolinux_name

end

if __FILE__ == $PROGRAM_NAME
  if ARGV.empty?
    # In this case, do some testing.
    n_ljust = 12
    _ = 'glib';         puts _.ljust(n_ljust)+' -> '+Cookbooks.gobolinux_naming_convention(_)
    _ = 'atk';          puts _.ljust(n_ljust)+' -> '+Cookbooks.gobolinux_naming_convention(_)
    _ = 'gcc';          puts _.ljust(n_ljust)+' -> '+Cookbooks.gobolinux_naming_convention(_)
    _ = 'mate-desktop'; puts _.ljust(n_ljust)+' -> '+Cookbooks.gobolinux_naming_convention(_)
    _ = 'libdrm';       puts _.ljust(n_ljust)+' -> '+Cookbooks.gobolinux_naming_convention(_)
  else
    puts Cookbooks.gobolinux_naming_convention(ARGV)
  end
end # gobolinux_naming_convention