#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/module_methods/all_binaries.rb'
# =========================================================================== #
require 'cookbooks/project_settings/project_yaml_directory.rb'
require 'cookbooks/requires/require_yaml.rb'

module Cookbooks

  # ========================================================================= #
  # === LOCATION_REGISTERED_BINARIES
  #
  # This may be a file path such as:
  #
  #   /Programs/Ruby/2.4.1/lib/ruby/site_ruby/2.4.0/cookbooks/yaml/registered_binaries.yml
  #
  # cat $COOKBOOKS/yaml/registered_binaries.yml
  # ========================================================================= #
  LOCATION_REGISTERED_BINARIES =
    Cookbooks.yaml_directory?+'registered_binaries.yml'

  # ========================================================================= #
  # === YAML_FILE_REGISTERED_BINARIES
  #
  # This is equivalent to:
  #   data = YAML.load_file(Cookbooks.yaml_directory?+'registered_binaries.yml')
  # ========================================================================= #
  if File.exist? LOCATION_REGISTERED_BINARIES
    YAML_FILE_REGISTERED_BINARIES = YAML.load_file(LOCATION_REGISTERED_BINARIES)
  else
    YAML_FILE_REGISTERED_BINARIES = {} # Otherwise, initialize as empty hash.
  end; ALL_BINARIES = YAML_FILE_REGISTERED_BINARIES # === ALL_BINARIES

  # ========================================================================= #
  # === Cookbooks.location_registered_binaries
  # ========================================================================= #
  def self.location_registered_binaries
    LOCATION_REGISTERED_BINARIES
  end

  # ========================================================================= #
  # === Cookbooks.all_binaries?
  #
  # Get back all registered binaries.
  #
  # This is guaranteed to return a Hash.
  # ========================================================================= #
  def self.all_binaries?
    ALL_BINARIES
  end

  # ========================================================================= #
  # === all_binaries?
  # ========================================================================= #
  def all_binaries?
    Cookbooks.all_binaries?
  end

end

if __FILE__ == $PROGRAM_NAME
  pp Cookbooks.all_binaries?
  puts 'This was read from the file: '+
       Cookbooks.location_registered_binaries
end # all_binaries?