#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/module_methods/all_libraries.rb'
# =========================================================================== #
require 'cookbooks/project_settings/project_yaml_directory.rb'
require 'cookbooks/requires/require_yaml.rb'

module Cookbooks

  # ========================================================================= #
  # === LOCATION_REGISTERED_LIBRARIES
  #
  # This may be a file path such as:
  #
  #   /Programs/Ruby/2.4.1/lib/ruby/site_ruby/2.4.0/cookbooks/yaml/registered_libraries.yml
  #
  # cat $COOKBOOKS/yaml/registered_libraries.yml
  # ========================================================================= #
  LOCATION_REGISTERED_LIBRARIES =
    Cookbooks.yaml_directory?+'registered_libraries.yml'

  # ========================================================================= #
  # === YAML_FILE_REGISTERED_LIBRARIES
  #
  # This is equivalent to:
  #   data = YAML.load_file(Cookbooks.yaml_directory?+'registered_libraries.yml')
  # ========================================================================= #
  if File.exist? LOCATION_REGISTERED_LIBRARIES
    YAML_FILE_REGISTERED_LIBRARIES = YAML.load_file(LOCATION_REGISTERED_LIBRARIES)
  else
    YAML_FILE_REGISTERED_LIBRARIES = {} # Otherwise, initialize as empty hash.
  end; ALL_LIBRARIES = YAML_FILE_REGISTERED_LIBRARIES # === ALL_LIBRARIES

  # ========================================================================= #
  # === Cookbooks.location_registered_libraries
  # ========================================================================= #
  def self.location_registered_libraries
    LOCATION_REGISTERED_LIBRARIES
  end

  # ========================================================================= #
  # === Cookbooks.all_libraries?
  #
  # Get back all registered libraries.
  #
  # This is guaranteed to return a Hash.
  # ========================================================================= #
  def self.all_libraries?
    ALL_LIBRARIES
  end

  # ========================================================================= #
  # === all_libraries?
  # ========================================================================= #
  def all_libraries?
    Cookbooks.all_libraries?
  end

end

if __FILE__ == $PROGRAM_NAME
  pp Cookbooks.all_libraries?
  puts 'This was read from the file: '+
       Cookbooks.location_registered_libraries
end # all_libraries?