#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
require 'cookbooks/class/class.rb'
require 'cookbooks/colours/colours.rb'
require 'cookbooks/constants/constants.rb'
require 'cookbooks/module_methods/e.rb'
require 'cookbooks/module_methods/opn.rb'
require 'cookbooks/module_methods/initialize_configuration.rb'

module Cookbooks

  # ========================================================================= #
  # === @html_page_content
  # ========================================================================= #
  @html_page_content = ''.dup

  # ========================================================================= #
  # === Cookbooks.create_html_page_for_these_programs
  #
  # This method works on an Array as its input. This Array should be
  # about programs that you want to see an auto-generated webpage
  # containing some information about this program.
  # ========================================================================= #
  def self.create_html_page_for_these_programs(array = ARGV)
    if array.is_a?(String) and
       array.start_with?(':')
      array = array.to_s.delete(':').to_sym
    end
    if array.is_a? Symbol
      unless Cookbooks.const_defined? :SearchForTags
        require 'cookbooks/utility_scripts/search_for_tags/search_for_tags.rb'
      end
      # ===================================================================== #
      # Handle special cases such as KDE5.
      # ===================================================================== #
      case array # case tag
      # ===================================================================== #
      # === cookbooks --create_html_page_for_these_programs=:kde5
      # ===================================================================== #
      when :kde, :kde5
        array = ::Cookbooks.return_all_programs_that_match_this_tag(:kde5)
      # ===================================================================== #
      # === cookbooks --create_html_page_for_these_programs=:kde5_toolchain
      # ===================================================================== #
      when :kde5_toolchain
        # =================================================================== #
        # This variant here should be equivalent to LFS/BLFS, in the proper
        # order. This thus requires a yaml file.
        # =================================================================== #
        dataset = YAML.load_file(FILE_COOKBOOK_CHAINED_PROGRAMS)
        array = dataset['kde5_foundation']+
                dataset['kde5_plasma']+
                dataset['kde5_applications']+
                dataset['kde5_porting_aids']
        array.delete!(' ')
        array = array.split(',')
      end
    end
    if array.is_a? Array
      if array.first.is_a?(String) and array.first.include?(',') # <- Input such as ["php,kio,ruby"]
        array = array.first.split(',')
      end
      array.flatten!
      if array.size > 20
        opn; e 'More than 20 entries were given (exactly '+
               sfancy(array.size.to_s)+'). This may take a while.'
      end
      @html_page_content = "<html><head><title>HTML Pages for programs</title>"\
                           "<style>pre { font-size: 1.4em; }</style>"\
                           "</head><body><hr>\n".dup # Reset again.
      array.each_with_index {|this_program, index| index += 1
        # =================================================================== #
        # For each program, we do the same actions:
        #   (1) Obtain the dataset via Cookbooks::Cookbook.new() OR
        #       from the expanded directory containing all yaml files.
        #   (2) 
        # =================================================================== #
        if File.directory?(DIRECTORY_EXPANDED_COOKBOOKS) and
           File.exist?(target_file = "#{DIRECTORY_EXPANDED_COOKBOOKS}#{this_program}.yml")
          dataset = YAML.load_file(target_file)
          remote_url  = dataset['url1']
          description = dataset['raw_description']
          licence     = dataset['licence']
          homepage    = dataset['homepage'].to_s
        else
          dataset = Cookbooks::Cookbook.new(this_program)
          remote_url  = dataset.url1?
          description = dataset.raw_description?
          licence     = dataset.licence?
          homepage    = dataset.homepage?
        end
        if licence.is_a? Array
          licence = licence.join(', ')
        end
        @html_page_content << "<p>\n<h3 style=\"color:darkgreen\"><b style=\"color:darkblue\">"\
                              "#{index})</b> #{this_program}</h3><br>\n"
        @html_page_content <<
          "  <span style=\"margin-left: 1em; font-weight:bold; color: darkblue\">wget #{remote_url}<span><br><br>\n"
        unless description.to_s.empty? # <- Ignore empty descriptions.
          @html_page_content << "<pre>#{description}</pre>\n"
        end
        @html_page_content << "licence: #{licence}<br><br>\n"
        @html_page_content << "homepage: <a href=\"#{homepage}\">#{homepage}</a><br>\n"
        @html_page_content << "</p><br><hr>\n"
      }
      # ===================================================================== #
      # Next, store that HTML page somewhere.
      # ===================================================================== #
      what = @html_page_content
      into = "#{log_dir?}programs.html"
      opn; e "Storing these #{sfancy(array.size)} programs into the file:"
      opn; e "  #{sfile(into)}"
      write_what_into(what, into)
      @html_page_content = ''.dup # Clear it again here.
    else
      opn; e 'This method requires an Array as input.'
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  input = ARGV
  # input = :kde5 # For debugging purposes.
  Cookbooks.create_html_page_for_these_programs(input)
end # create_html_page_for_these_programs solid phonon plasmaframework kwin