#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/module_methods/create_program_version_url_file.rb'
# =========================================================================== #
require 'cookbooks/module_methods/initialize_configuration.rb'
require 'cookbooks/module_methods/opn.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks.create_program_version_url_file
  #
  # This method will create a local .txt file that will show the name
  # of a program, the version and the remote (first) URL to the
  # program at hand.
  #
  # The boolean argument that can be passed to this method determines
  # as to whether we will also show the time when the program has
  # been last updated. By default this is true, meaning that we
  # WILL show when a program has been last updated.
  #
  # To invoke this method from the commandline, do:
  #   cookbooks --create_program_version_url_file
  # ========================================================================= #
  def self.create_program_version_url_file(
      also_show_last_update = :default # Since Feb 2018, we will show the last update by default.
    )
    case also_show_last_update # case tag
    when :also_show_last_update,
         :include_last_update,
         :default
      also_show_last_update = true
    end
    newline = "\n".dup
    array_to_be_stored = []
    _ = DIRECTORY_EXPANDED_COOKBOOKS
    if File.directory?(_)
      all_cookbooks = Dir["#{_}*.yml"].sort
      all_cookbooks.each {|yaml_file|
        dataset = YAML.load_file(yaml_file)
        program_name    = File.basename(yaml_file.sub(/\.yml$/,''))
        program_version = dataset['program_version']
        program_url     = dataset['url1']
        array_to_be_stored << [
          program_name, program_version, program_url
        ]
        if also_show_last_update
          # ================================================================= #
          # In this case, append more information.
          # ================================================================= #
          last_update = dataset['last_update']
          array_to_be_stored[-1] << last_update 
        end
      }
      # ===================================================================== #
      # Now that we have populated array_to_be_stored, we must
      # use it to create a .txt file.
      # ===================================================================== #
      what = ''.dup
      array_to_be_stored.each {|entry|
        program_name, program_version, program_url = entry[0], entry[1], entry[2]
        if also_show_last_update
          last_update = entry[3] # It is the last element.
        end
        # =================================================================== #
        # Add what we need here - name of the program, the version and
        # the remote URL. This will be properly padded.
        # =================================================================== #
        what << program_name.to_s.ljust(28)+
                program_version.to_s.ljust(22)
        # =================================================================== #
        # Append when the last update happened, if we use that entry at
        # all.
        # =================================================================== #
        if also_show_last_update
          what << last_update.to_s.strip.ljust(14)
        end
        # =================================================================== #
        # Next, append the remote URL to the line.
        # =================================================================== #
        what << program_url.to_s.rstrip.ljust(44).rstrip+
                newline
      }
      into = "#{Cookbooks.log_dir?}CookbookDirectory/programs_version_url.txt"
      opn; e 'Now storing into `'+sfile(into)+'`.'
      write_what_into(what, into)
    else
      # all_programs = Cookbooks.available_programs?
      opn; e 'Currently, this functionalitye depends on the expanded '\
             'cookbook dataset.'
      opn; e 'Generate it anew via:'
      opn; e '  cookbooks --expand'
    end
  end; self.instance_eval { alias generate_programs_version_url_yaml_file create_program_version_url_file } # === Cookbooks.generate_programs_version_url_yaml_file

end

if __FILE__ == $PROGRAM_NAME
  # Cookbooks.create_program_version_url_file
  Cookbooks.create_program_version_url_file(:also_show_last_update)
end # create_program_version_url_file