#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
require 'cookbooks/class/class.rb'
require 'cookbooks/misc/available_cookbooks.rb'
require 'cookbooks/module_methods/e.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks.enable_module_programs
  #
  # This method will be invoked when we require the cookbooks project
  # in a special manner.
  #
  # We add the individual cookbooks onto the Cookbooks. namespace directly
  # there.
  #
  # This will then allow us to call methods such as:
  #
  #   Cookbooks.htop
  #
  # Do note that we will also radically override already existing methods,
  # so this is functionality that should be used with care.
  #
  # Testing this method can happen either like this:
  #
  #   Cookbooks.enable_module_programs; Cookbooks.htop
  #   require 'cookbooks/module_programs'; htop = Cookbooks.htop
  # ========================================================================= #
  def self.enable_module_programs
    available_programs?.each {|this_program|
      self.class.class_eval {
        _ = this_program.to_sym
        if respond_to? _
          # This here happens when we enable the methods.
          # This is just here as a placeholder, if we ever want
          # to debug this.
        else
          # ::Cookbooks.e 'Defining the method `'+_.to_s+'` next.' # <- This is more o a debug-statement.
          define_method(_) { # Simply delegate to class Cookbook then.
            ::Cookbooks::Cookbook.new(this_program)
          }
        end
      }
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks.enable_module_programs
  pp Cookbooks.htop
end