#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/module_methods/ftp.rb'
# =========================================================================== #
require 'cookbooks/module_methods/opn.rb'
require 'cookbooks/module_methods/initialize_configuration.rb'
require 'cookbooks/module_methods/time.rb'
require 'cookbooks/misc/save_file.rb'
require 'cookbooks/module_methods/create_programs_version_html_file.rb'
begin
  require 'ftp_paradise'
rescue LoadError; end

module Cookbooks

# =========================================================================== #
# === Cookbooks.publish_list_of_all_programs_version
#
# To invoke this method, do:
#
#   cookbooks --publish-list-of-all-programs-version
#
# =========================================================================== #
def self.publish_list_of_all_programs_version(
    file_programs_version = FILE_PROGRAMS_VERSION # This is the file 'programs_version.yml'

  )
  _ = ''.dup
  opn; e 'The file '+sfile(file_programs_version)+' will next '\
          'be FTP-uploaded/published.'
  hash = YAML.load_file(file_programs_version)
  hash.each_pair {|name_of_the_program, program_version|
    _ << name_of_the_program.ljust(28)+program_version.to_s+N
  }
  what = _
  into = Cookbooks.store_into_this_directory+'programs_version.txt'
  opn; e 'Storing into `'+sfile(into)+'`.'
  ::Cookbooks::SaveFile.write_what_into(what, into)
  ::Cookbooks.upload_list_of_all_programs_version(into)
  # ========================================================================= #
  # Also upload a .html variant.
  # ========================================================================= #
  ::Cookbooks.upload_list_of_all_programs_version(into, :html)
end

# =========================================================================== #
# === Cookbooks.upload_list_of_all_programs_version
#
# This will use my FTP wrapper to upload this file.
#
# The local .txt file is usually kept or autogenerated here:
#   /Depot/Temp/CookbookDirectory/programs_version.txt
# =========================================================================== #
def self.upload_list_of_all_programs_version(
    this_file = Cookbooks.store_into_this_directory+'programs_version.txt',
    type      = :txt # type can be :txt or :html
  )
  if File.exist? this_file
    case type
    when :html
      ::Cookbooks.create_programs_version_html_file(this_file)
      this_file.sub!(/\.txt$/,'')
    end
    opn; e 'Uploading the file '+sfile(this_file)+' next.'
    if Object.const_defined? :FtpParadise
      _ = FtpParadise::Connection.new(:dont_run_yet)
      _.set_be_verbose
      _.set_mode :ascii
      _.set_data('SHEVY')
      _.do_login('SHEVY')
      _.upload(this_file, :be_silent)
    end
  else
    opn; e 'No file exists at `'+sfile(this_file)+'`.'
  end
end

end

if __FILE__ == $PROGRAM_NAME
  # The next line is mostly for standalone-testing of this file here.
  Cookbooks.upload_list_of_all_programs_version
end