#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/module_methods/is_included.rb'
# =========================================================================== #
require 'cookbooks/aliases/aliases.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks.find_cookbook_alias_for
  #
  # Usage example:
  #   Cookbooks.find_cookbook_alias_for 'perlx'
  # ========================================================================= #
  def self.find_cookbook_alias_for(i)
    Cookbooks::Aliases.new(i).value?
  end; self.instance_eval { alias alias?         find_cookbook_alias_for } # === Cookbooks.find_cookbook_alias_for
       self.instance_eval { alias find_alias_for find_cookbook_alias_for } # === Cookbooks.find_alias_for

  # ========================================================================= #
  # === Cookbooks.is_an_alias?
  #
  # This method will return true if the input is an alias, and otherwise
  # it will return false.
  #
  # We will get rid of ':' in the input.
  #
  # To test this method, do:
  #
  #   Cookbooks.is_an_alias? 'htop'
  #   Cookbooks.is_an_alias? 'blkid'
  # ========================================================================= #
  def self.is_an_alias?(i)
    _ = Cookbooks::Aliases.new(i)
    _.is_an_alias?
  end

  # ========================================================================= #
  # === Cookbooks.is_included?
  #
  # This method can be used to query whether a given input is included
  # or whether it is not.
  #
  # By default we will also include aliases, but this can be disabled,
  # such as via the Symbol :do_not_check_for_aliases.
  #
  # Remember that all aliases in use are stored in a yaml file. If you
  # want to add new aliases or change the existing ones, you have to
  # modify that yaml file.
  #
  # In principle, we could also restrict this method here to only-aliases
  # but I found that the API reads differently than this intent.
  #
  # Usage examples:
  #
  #    Cookbooks.is_included? 'abiword'
  #    Cookbooks.is_included? 'aclocal'
  # 
  # ========================================================================= #
  def self.is_included?(
      i,
      honour_aliases = true
    )
    if i
      i = i.join(' ').strip if i.is_a? Array
      i.delete!(':') if i.include? ':' # We ignore ':' tokens.
      keys = Cookbooks.available_programs?
      case honour_aliases
      when :default # Restore the default state in this case.
        honour_aliases = true
      when :do_not_honour_aliases,
           :do_not_check_for_aliases,
           :do_not_use_aliases,
           :ignore_aliases,
           :user_disabled_this_setting_via_the_commandline
        honour_aliases = false
      end
      if honour_aliases
        aliases = Cookbooks::Aliases.new(i)
        all_aliases = aliases.dataset
        # =================================================================== #
        # Merge the keys together next, starting with the default programs.
        # ==================================================================== #
        keys << all_aliases.keys
      end
      keys.flatten!
      return keys.include?(i)
    else
      false
    end
  end; self.instance_eval { alias really_does_include? is_included? } # === Cookbooks.really_does_include?
       self.instance_eval { alias does_include?        is_included? } # === Cookbooks.does_include?
       self.instance_eval { alias program_is_included? is_included? } # === Cookbooks.program_is_included?

end

if __FILE__ == $PROGRAM_NAME
  require 'colour_e/autoinclude'
  require 'verbose_truth/autoinclude'
  e 'Is the program given by '+ARGV.first+' included? '+
    sfancy(VerboseTruth[Cookbooks.program_is_included?(ARGV)])
end # program_is_included?
    # program_is_included? gtk