# =========================================================================== #
# require 'cookbooks/module_methods/permanently_set_source_directory.rb'
# =========================================================================== #
require 'cookbooks/constants/misc.rb'
require 'cookbooks/colours/colours.rb'
require 'cookbooks/module_methods/e.rb'
require 'cookbooks/module_methods/opn.rb'
require 'cookbooks/misc/save_file.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks.permanently_set_source_directory
  #
  # Use this method if you wish to permanently modify the source
  # directory, that is, the directory where our archives will be
  # stored.
  #
  # Invocation example from the commandline:
  #   cookbooks --set_source_dir=/opt
  #  cookbooks --permanently-set-base-directory-to=/opt
  # ========================================================================= #
  def self.permanently_set_source_directory(i)
    i << '/' unless i.end_with? '/'
    # ======================================================================= #
    # Modify the yaml file next:
    # ======================================================================= #
    what = i
    into = FILE_SOURCE_DIRECTORY_YAML
    ::Cookbooks::SaveFile.write_what_into(what, into)
    opn; e 'Setting the new source directory to point towards '+sdir(what)+'.'
    opn; e 'Storing into the file `'+sfile(into)+'` next.'
    # ======================================================================= #
    # And also modify it on my home system, if IS_ROEBE is true. This will
    # go right into the configuration/ subirectory.
    # ======================================================================= #
    if is_on_roebe?
      into = RUBY_SRC_DIR_AT_HOME+'cookbooks/lib/cookbooks/yaml/configuration/'+
             File.basename(into)
      opn; e 'Will also store into the file `'+sfile(into)+'`.'
      ::Cookbooks::SaveFile.write_what_into(what, into)
    end
    Cookbooks.set_source_directory(i) # And also sync it back.
  end

end