#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/module_methods/report_extended_information_about_this_program.rb'
# =========================================================================== #
require 'cookbooks/class/cookbooks.rb'
require 'cookbooks/module_methods/opn.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks.check_and_report_whether_file_exists
  #
  # The first argument to this method must be the path to the archive.
  # ========================================================================= #
  def self.check_and_report_whether_file_exists(
      i,
      optional_cookbook_object = nil
    )
    i = i.to_s
    unless File.exist? i
      opnn; Colours.ewarn 'This archive does not exist locally, at'
      opnn; Colours.ewarn '  `'+Colours.sfile(i)+swarn('`')
      if optional_cookbook_object
        if optional_cookbook_object.do_not_download?
          opnn; e 'This is, however had, no problem as it will not'
          opnn; e 'be downloaded anyway.'
        end
      end
      cliner :lightblue
    end
  end

  # ========================================================================= #
  # === Cookbooks.report_extended_information_about_this_program
  #
  # (feedback tag, debug tag, main tag)
  #
  # Use this feedback-method to report extended information about a
  # program, in a colourful manner. This program must have been registered
  # in the Cookbooks namespace - otherwise we can not really display
  # much about that program at hand.
  #
  # The input to this method can be either be a String or an Array.
  # ========================================================================= #
  def self.report_extended_information_about_this_program(
      i
    )
    i = i.first if i.is_a? Array
    if i
      _ = Cookbooks::Cookbook.new
      _.find_this_program(i)
      _.report
      check_and_report_whether_file_exists(_.full_path?, _)
    end
  end; self.instance_eval { alias feedback_colourized_result report_extended_information_about_this_program } # === Cookbooks.feedback_colourized_result

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks.report_extended_information_about_this_program(ARGV)  
end