#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/module_methods/report_total_size_of_all_archives.rb'
# =========================================================================== #
require 'cookbooks/misc/available_cookbooks.rb'
require 'cookbooks/module_methods/initialize_configuration.rb'
require 'cookbooks/colours/colours.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks.report_total_size_of_all_archives
  #
  # This method can be used to report the total size of all locally
  # available archives.
  #
  # Invocation example from the commandline:
  #   cookbooks --report_total_size_of_all_archives
  # ========================================================================= #
  def self.report_total_size_of_all_archives
    total_size = 0
    available_programs = available_programs?
    available_programs.each {|this_program|
      possible_target = "#{DIRECTORY_EXPANDED_COOKBOOKS}#{this_program}.yml"
      if File.exist? possible_target
        dataset = YAML.load_file(possible_target)
        program_path = dataset['program_path']
      else
        # =================================================================== #
        # The expanded-cookbooks file does not exist, so use
        # Cookbooks::Cookbook instead.
        # =================================================================== #
        unless Cookbooks.const_defined? :Cookbook
          require 'cookbooks/class/class.rb'
        end
        program_path = Cookbooks::Cookbook.new(this_program).program_path?
      end
      if File.exist? program_path
        total_size += File.size(program_path)
      end
    }
    n_kilobytes = total_size  / 1024.0
    n_megabytes = n_kilobytes / 1024.0
    n_gigabytes = n_megabytes / 1024.0
    # ======================================================================= #
    # Finally, report it here.
    # ======================================================================= #
    e 'The total file size of all '+sfancy(available_programs.size.to_s)+
      ' programs is '+total_size.to_s+' bytes '
    e "(#{n_kilobytes.round(2)} KB; "+
      n_megabytes.round(2).to_s+' MB; '+
      crimson(n_gigabytes.round(2).to_s+' GB')+')'
  end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks.report_total_size_of_all_archives
end # report_total_size_of_all_archives