#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/module_methods/swift_return_version_of_this_program.rb'
# =========================================================================== #
require 'cookbooks/aliases/aliases.rb'
require 'cookbooks/constants/constants.rb'
require 'cookbooks/requires/require_yaml.rb'
require 'cookbooks/errors/file_does_not_exist.rb'
require 'cookbooks/module_methods/is_included.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks.swift_return_version_of_this_program
  #
  # This method here is different from Cookbooks.return_version_of_this_program()
  #
  # Rather than instantiate a new Cookbooks::Cookbook object, we just query
  # the version from a yaml file directly. This is obviously much faster.
  #
  # The obvious drawback is that we may have to manually update that yaml
  # file and keep it up-to-date - but the advantage by far outweights this
  # disadvantage. This method here will be much faster than
  # Cookbooks.return_version_of_this_program() - and ideally, you should
  # use it, rather than the other method.
  #
  # If the program in question can not be found, this method will return
  # nil.
  #
  # Invocation examples:
  #   Cookbooks.swift_return_version_of_this_program(:ruby) # => "2.3.1"
  #   Cookbooks.swift_return_version_of_this_program(:htop) # => "2.0.2"
  #   Cookbooks.swift_return_version_of_this_program(:awk)  # => "2.0.2"
  # ========================================================================= #
  def self.swift_return_version_of_this_program(
      this_program = :htop
    )
    this_program = this_program.to_s
    # ======================================================================= #
    # Next, try to pass it against a list of aliases, if it has not
    # been yet "findable".
    # ======================================================================= #
    if !Cookbooks.does_include?(this_program, :ignore_aliases)
      possible_alternative_name = Cookbooks.find_cookbook_alias_for(this_program)
      if possible_alternative_name
        if possible_alternative_name != this_program
          this_program = possible_alternative_name
        end
      end
    end
    _ = FILE_PROGRAMS_VERSION
    if File.exist? _
      dataset = YAML.load_file(_)
      if dataset.has_key? this_program
        program_version = dataset.fetch(this_program)
        return program_version
      end
    else
      e "No file called `#{sfile(_)}` exists."
      raise FileDoesNotExist, _.to_s+' does not exist.'
    end
    return nil # Default return value here.
  end; self.instance_eval { alias swift_return_version swift_return_version_of_this_program } # === Cookbooks.swift_return_version
       self.instance_eval { alias return_version_of    swift_return_version_of_this_program } # === Cookbooks.return_version_of

end

if __FILE__ == $PROGRAM_NAME
  puts 'Awk version:   '+Cookbooks.swift_return_version_of_this_program(:awk).to_s # => "2.0.2"
  puts 'Glibc version: '+Cookbooks.swift_return_version_of_this_program(:glibc).to_s
  puts 'Atk version:   '+Cookbooks.swift_return_version_of_this_program(:atk).to_s
end