#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# require 'cookbooks/module_methods/update_all_ruby_gems.rb'
# =========================================================================== #
require 'cookbooks/misc/available_cookbooks.rb'
require 'cookbooks/directory/individual_cookbooks_directory.rb'
require 'cookbooks/module_methods/e.rb'
require 'cookbooks/check_for_updates/check_for_new_release_on_rubygems.rb'

module Cookbooks

  # ========================================================================= #
  # === Cookbooks.update_all_ruby_gems
  #
  # This method can be used to update all local ruby gems (.gem files).
  # ========================================================================= #
  def self.update_all_ruby_gems
    _ = Cookbooks::CheckForNewReleaseOnRubygems.new(:dont_run_yet)
    base_dir = Cookbooks.individual_cookbooks_directory?
    # ======================================================================= #
    # First, we must obtain all entries thave have the strings "url1"
    # and ".gem". We will ask ruby to perform a grep-operation. We will
    # store these in an array.
    # ======================================================================= #
    array_ruby_gems = []
    available_programs = Cookbooks.available_programs?
    available_programs.each {|this_program|
      target_yaml_file = "#{base_dir}#{this_program}.yml"
      if File.exist? target_yaml_file
        dataset = File.read(target_yaml_file)
        if dataset.include?('url1')
          line_holding_url1 = dataset.scan(/^ url1: (.+)$/).flatten.first
          if line_holding_url1.end_with? '.gem' # Found ruby gems.
            array_ruby_gems << [this_program, line_holding_url1] 
          end
        end
      end
    }
    # ======================================================================= #
    # Ok - now that we have obtained the files that are .gem files,
    # we can proceed them and check for an updated. This taps into
    # a specialized class that will do so for us.
    # ======================================================================= #
    array_ruby_gems.each {|this_ruby_gem, remote_url|
      _.check_this_remote_url(remote_url)
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks.update_all_ruby_gems
end # update_all_ruby_gems