#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# ============================================================================ #
# require 'cookbooks/requires/require_yaml.rb'
# ============================================================================ #
require 'cookbooks/constants/configuration_constants.rb'
require 'yaml'

# ============================================================================ #
# We will also enable a custom variant to load yaml files.
# ============================================================================ #
module Cookbooks

  # ========================================================================== #
  # === Cookbooks.yaml_engine?
  # ========================================================================== #
  def self.yaml_engine?
    YAML.load_file(Cookbooks::FILE_USE_PSYCH_OR_SYCK)
  end

  case self.yaml_engine? # case tag
  when :psych, :default
    # ======================================================================= #
    # pass through because psych is the default yaml engine.
    # ======================================================================= #
  when :syck
    begin
      require 'syck'
    rescue LoadError; end
    begin
      YAML::ENGINE.yamler = 'syck'
    rescue NameError; end
  end

  # ========================================================================== #
  # === Cookbooks.load_yaml
  #
  # Use this when loading a yaml file in the Cookbooks project.
  # ========================================================================== #
  def self.load_yaml(i)
    YAML.load_file(i)
  end

  # ========================================================================== #
  # === load_yaml
  # ========================================================================== #
  def load_yaml(i)
    ::Cookbooks.load_yaml(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  pp Cookbooks.yaml_engine?
end