#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::AllowedCookbookEntries
#
# The purpose of this class is to simply show to the user which cookbook
# entries are legal/valid and which ones are not.
#
# You can query this information from the commandline as well, by
# issuing:
#
#   cookbooks --allowed-cookbook-entries
#
# =========================================================================== #
# require 'cookbooks/utility_scripts/allowed_cookbook_entries.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/constants/registered_cookbook_entries.rb'
require 'cookbooks/module_methods/e.rb'
require 'cookbooks/colours/colours.rb'

module Cookbooks

class AllowedCookbookEntries < Base # === Cookbooks::AllowedCookbookEntries

  # ========================================================================= #
  # === Cookbooks::AllowedCookbookEntries::COLOURIZE_THE_FIRST_CHARACTER
  # ========================================================================= #
  COLOURIZE_THE_FIRST_CHARACTER = true

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(run_already = true)
    super()
    reset
    run if run_already
  end

  # ========================================================================= #
  # === Cookbooks::AllowedCookbookEntries.e
  # ========================================================================= #
  def self.e(i = '')
    ::Cookbooks.e(i)
  end

  # ========================================================================= #
  # === Cookbooks::AllowedCookbookEntries.report_allowed_entries
  # ========================================================================= #
  def self.report_allowed_entries
    e; opnn; e 'The following entries are registered in the file '
    opnn; e ::Cookbooks.sfile('registered_cookbook_entries.rb')+', in the '
    opnn; e 'constant '+::Cookbooks.simp('REGISTERED_COOKBOOK_ENTRIES')+', and thus '\
            'valid:'; e
    REGISTERED_COOKBOOK_ENTRIES.each_with_index {|entry, index|
      index += 1
      if COLOURIZE_THE_FIRST_CHARACTER
        entry[0,1] = ::Cookbooks.swarn(entry[0,1]) # Make the first character red.
      end
      e ::Cookbooks.sfancy(index.to_s.rjust(3))+') '+entry
    }; e # Need to output a terminating-newline here for consistency with the above.
  end

  # ========================================================================= #
  # === AllowedCookbookEntries.opnn
  # ========================================================================= #
  def self.opnn
    Opn.opn(use_this_as_namespace: NAMESPACE)
  end

  # ========================================================================= #
  # === AllowedCookbookEntries[]
  # ========================================================================= #
  def self.[](i = '')
    self.new
  end

  # ========================================================================= #
  # === run
  #
  # We redirect to a class method here.
  # ========================================================================= #
  def run
    AllowedCookbookEntries.report_allowed_entries # Call the class method here.
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::AllowedCookbookEntries.new
end # acookbooks