#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# =========================================================================== #
# === Cookbooks::Convert_PKGBUILD
#
# This class converts a pkgbuild into a valid Cookbooks archive. It does so
# by first parsing the PKGBUILD in question, then loading this information
# as a SLoad object. Then you can generate a yaml file from this data.
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb' 

module Cookbooks

class Convert_PKGBUILD < Cookbooks::Base

  # ========================================================================= #
  # === DEFAULT_FILE
  # ========================================================================= #
  DEFAULT_FILE =
    '/Users/x/DATA/PC/OS/LINUX/DISTRIBUTIONEN/ARCH/example.pkgbuild'

  # ========================================================================= #
  # === HASH_MAP_PKGBUILD_VARIABLE_TO_COOKBOOK_ENTRY
  #
  # Use a mapping, to map from PKGBUILD variables to our Cookbook entries.
  # ========================================================================= #
  HASH_MAP_PKGBUILD_VARIABLE_TO_COOKBOOK_ENTRY = {
    'makedepends'  => 'required_deps_on',
    'pkgver'       => 'program_version', # The version of the package.
    'url'          => 'url1',
    'license'      => 'license',
    'pkgname'      => 'program_name', # This is program_name.
    'pkgdesc'      => 'description',
    'depends'      => 'required_deps_on',
    'builddepends' => 'build_deps_on',
    'optdepends'   => 'recommended_deps_on'
  }; H = HASH_MAP_PKGBUILD_VARIABLE_TO_COOKBOOK_ENTRY # Easier alias.

  # ========================================================================= #
  # The following entries have no corresponding version.
  #
  # epoch is an integer value, specific to Arch Linux, representing what
  # 'lifetime' to compare version numbers against. This value allows
  # overrides of the normal version comparison rules for packages that
  # have inconsistent version numbering, require a downgrade, change
  # numbering schemes, etc. By default, packages are assumed to have an
  # epoch value of 0. Do not use this unless you know what you are doing.
  # - arch is an array of architectures that the PKGBUILD file is known
  # to build and work on. Currently, it should contain i686 or x86_64
  # or any for architecture-independent packages.
  # ========================================================================= #
  ARRAY_ENTRIES_WITHOUT_CORRESPONDING_VERSIONS = %w(
    pkgrel
    epoch
  )
  # ========================================================================= #
  # Arch has no corresponding entry.
  # ========================================================================= #

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_pkgbuild = DEFAULT_FILE,
      run_already   = true
    )
    super()
    reset
    read_data_from(this_pkgbuild)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @query = nil
    @hash = {}
  end

  # ========================================================================= #
  # === read_data_from
  #
  # Sets @data variable.
  # ========================================================================= #
  def read_data_from(i = DEFAULT_FILE)
    @data = File.readlines(i).map(&:chomp)
  end

  # ========================================================================= #
  # === process_data
  #
  # We work through @data here.
  # ========================================================================= #
  def process_data
    @data.each { |line|
      case line
      when /^(pkgname)/, /^(pkgver)/, /^(pkgdesc)/,
           /^(url)/,/^(license)/,/^(depends)/,
           /^(makedepends)/
        @hash[ H[$1] ] = split(line)
      end
    }
    _ = Cookbooks::Cookbook.new(@hash['program_name']) { :bypass_menu }
    _.clear(false)
    _.merge(@hash)
    _.set_archive_type('.tar.gz') # hardcoded for now.
pp _
pp "STILL NOT FINISHED. SORRY."
  end

  # ========================================================================= #
  # === remove_parens
  #
  # This method will turn something like ('FOO') into 'FOO'
  # ========================================================================= #
  def remove_parens(i)
    i.delete('()')
  end

  # ========================================================================= #
  # === split
  #
  # Easier to use this method to split at = characters found.
  # ========================================================================= #
  def split(i)
    i.split('=')[1]
  end

  # ========================================================================= #
  # === hash?
  # ========================================================================= #
  def hash?
    @hash
  end; alias hash hash? # === hash

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    process_data
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _ = Cookbooks::Convert_PKGBUILD.new
  pp _.hash
end # cpkg