#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# =========================================================================== #
# === Cookbooks::CreateDebianPackage
#
# This class will create a debian package.
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'
require 'cookbooks/module_methods/initialize_configuration.rb'

module Cookbooks

class CreateDebianPackage < Cookbooks::Base

  # ========================================================================= #
  # === CREATE_DEBIAN_PACKAGE_WHERE
  #
  # Here set the folder, where you wanna create a debian package.
  # ========================================================================= #
  CREATE_DEBIAN_PACKAGE_WHERE = ::Cookbooks.temp_dir?+'debian/'

  # ========================================================================= #
  # === initialize
  #
  # The first argument must be a hash.
  # ========================================================================= #
  def initialize(
      main_hash = nil,
      run_already = true
    )
    super()
    set_data(main_hash)
    run if run_already
  end

  # ========================================================================= #
  # === set_data
  #
  # The input should be a @hash. If it is omitted then we will simply
  # use SanitizeCookbookDataset.
  # ========================================================================= #
  def set_data(i = nil)
    if i.nil? # If this is the case, then we load the data on our own.
      i = Cookbooks::Cookbook.new('htop')
      i = i.data?
    end
    @data = i
    set_name(@data.fetch('short_name'))
  end

  # ========================================================================= #
  # === set_name
  # ========================================================================= #
  def set_name(i)
    @name = i
  end

  # ========================================================================= #
  # === create_debian_package                                       (deb tag)
  #
  # This is a wrapper towards all things needed to create a complete .deb
  # package.
  # ========================================================================= #
  def create_debian_package(data_file = '')
    create_debian_data_file(data_file) # creates data.tar.gz
    create_debian_binary               # creates debian-binary
    create_debian_control_package      # creates control.tar.gz
  end

  # ========================================================================= #
  # === create_debian_binary
  #
  # This (internally used) method will create the 
  #   debian-binary
  # file. Right now the allowed version is only 2.0
  # ========================================================================= #
  def create_debian_binary(which_version = '2.0')
    _ = CREATE_DEBIAN_PACKAGE_WHERE
    debian_binary = _+'debian-binary'
    cii ' => Next creating debian-binary file in folder '+sfile(_)
    if File.exist? _
      save_file(which_version, debian_binary)
    else
      opn; warn ' => Warning, dir '+debian_binary+'does not exist!'
      opn; warn 'Exiting now.'
      exit
    end
  end

  # ========================================================================= #
  # === create_debian_data_file
  # ========================================================================= #
  def create_debian_data_file
  end

  # ========================================================================= #
  # === name?
  # ========================================================================= #
  def name?
    @name
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run # run tag.
    cliner {
      opn; e ' => Now creating a debian package for `'+name?+'`.'
    }
    # Fine tuning. Code can be found in: bl $COOKBOOKS/rpackages.rb
    # BUILD_TOOLS/lib/bt/lib/POSSIBLY_DEPRECATED_packages.rb
    # Hmm. Should be reworked slightly.
    create_debian_data_file
    Packages.create_debian_data_file(select_from_package_dir)
    Packages.create_debian_binary
    Packages.create_debian_control_package(
      @data['program_name'],
      @data['program_version'],
      @data['description'],
      'optional','all','no',
      @data['required_deps_on'],
      'standard'
    )
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::CreateDebianPackage.new
end # cdeb