#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::CreateGlobalYamlDatabase
#
# This class will fetch the information from ALL the individual
# .yml files and create a giant .yml file with all the information
# from that.
# =========================================================================== #
# require 'cookbooks/utility_scripts/create_global_yaml_database.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'
require 'cookbooks/module_methods/initialize_configuration.rb'

module Cookbooks

class CreateGlobalYamlDatabase < Base # === Cookbooks::CreateGlobalYamlDatabase

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === STORE_HERE
  # ========================================================================= #
  STORE_HERE =
    "#{::Cookbooks.store_into_this_directory?}global_yaml_database.yml"
  
  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @dataset = Hash.new # This is the main hash.
  end

  # ========================================================================= #
  # === encoding?
  # ========================================================================= #
  def encoding?
    USE_THIS_ENCODING
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === CreateGlobalYamlDatabase.store_where?
  # ========================================================================= #
  def self.store_where?
    STORE_HERE
  end

  # ========================================================================= #
  # === store_into_yaml_file
  # ========================================================================= #
  def store_into_yaml_file
    into = STORE_HERE
    opnn; e "Now storing into `#{sfile(into)}`."
    if File.exist? into
      File.delete(into) # Always get rid of the old yaml file first.
    end
    what = YAML.dump(@dataset)
    save_what_into(what, into)
  end

  # ========================================================================= #
  # === obtain_all_programs
  # ========================================================================= #
  def obtain_all_programs
    @all_programs = Cookbooks.available_programs
  end

  # ========================================================================= #
  # === iterate_over_every_program
  # ========================================================================= #
  def iterate_over_every_program
    @all_programs.each_with_index {|program, index|
      dataset = Cookbooks::Cookbook.new(program) { :bypass_menu }
      dataset = dataset.dataset?
      hash = {}
      program = program.encode(encoding?)
      hash[program] = dataset
      _ = {}
      dataset.each_pair {|key, value|
        if key
          key = key.encode(encoding?)
          begin
            value = value.encode(encoding?) if value.is_a? String
          rescue Encoding::InvalidByteSequenceError
            opnn; e swarn('Error for '+simp(value))
            exit
          end
          _[key] = value
        end
      }
      dataset = _
      opnn; e "Now working on the program #{simp(program)}."
      @dataset.merge!(hash)
    }
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    obtain_all_programs
    iterate_over_every_program
    store_into_yaml_file
  end

end

# =========================================================================== #
# === Cookbooks.generate_global_yaml_database
# =========================================================================== #
def self.generate_global_yaml_database
  Cookbooks::CreateGlobalYamlDatabase.new
end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::CreateGlobalYamlDatabase.new
end # global_database