#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::FeedbackBinariesOf
#
# This class will yield to us all binaries of a given program.
# =========================================================================== #
# require 'cookbooks/utility_scripts/feedback_binaries_of.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'

module Cookbooks

class FeedbackBinariesOf < Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = 'ruby',
      run_already = true
    )
    super()
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i)
    i = i.join(' ').strip if i.is_a? Array
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    obtain_dataset
    @binaries = @dataset.binaries
    report_binaries
  end

  # ========================================================================= #
  # === obtain_dataset
  # ========================================================================= #
  def obtain_dataset
    @dataset = Cookbooks::Cookbook.new(input?)
  end

  # ========================================================================= #
  # === binaries?
  # ========================================================================= #
  def binaries?
    @binaries
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === report_binaries
  # ========================================================================= #
  def report_binaries
    if binaries?.empty?
      opnn; e 'We do not know whether `'+sfancy(search_term?)+
              '` will install any binaries.'
    else
      @binaries.each {|entry|
        e '  - '+entry
      }
    end
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::FeedbackBinariesOf.new(ARGV)
end # binaries_of? ruby