#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::FindMultipleVersions
#
# This class will traverse through the SOURCE_DIRECTORY directory,
# enter each directory and check whether we have multiple versions
# of a program.
#
# This allows us to find multiple versions that we may no longer want
# to backup or keep.
#
# Usage examples:

#   Cookbooks::FindMultipleVersions.new
# =========================================================================== #
# require 'find_multiple_versions.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'

module Cookbooks

class FindMultipleVersions < Base # === Cookbooks::FindMultipleVersions

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === ARRAY_IGNORE_THESE_DIRECTORIES
  #
  # These directories will be ignored.
  # ========================================================================= #
  ARRAY_IGNORE_THESE_DIRECTORIES = %w(
    windows
  )

  # ========================================================================= #
  # === LOCATION
  # ========================================================================= #
  LOCATION = ::Cookbooks.source_directory? # base.rb will have this constant.

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === find_all_directories_with_more_than_one_program
  # ========================================================================= #
  def find_all_directories_with_more_than_one_program
    result = Dir[LOCATION+'*'].reject {|entry| !File.directory? entry}
    # ======================================================================= #
    # Next, we have to filter out entries that we do not want to have.
    # ======================================================================= #
    unless ARRAY_IGNORE_THESE_DIRECTORIES.empty?
      result.reject! {|entry|
        ARRAY_IGNORE_THESE_DIRECTORIES.include? File.basename(entry)
      }
    end
    result.sort! # Keep it sorted alphabetically.
    result.map! {|entry| (entry << '/').squeeze('/') }
    return result
  end; alias all_dirs? find_all_directories_with_more_than_one_program # === all_dirs?

  # ========================================================================= #
  # === seek_and_report_directories_with_more_than_one_entry
  # ========================================================================= #
  def seek_and_report_directories_with_more_than_one_entry
    counter = 0
    all_dirs?.each {|dir|
      _ = Dir[dir+'*']
      if _.size > 1
        counter += 1
        opnn; e '('+sfancy(counter.to_s)+') The directory at `'+sdir(dir)+
                '` has more than one entry.'
        pp _
        e
      end
    }
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    seek_and_report_directories_with_more_than_one_entry
  end

end

# =========================================================================== #
# === Cookbooks.find_multiple_versions
# =========================================================================== #
def self.find_multiple_versions
  Cookbooks::FindMultipleVersions.new
end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks.find_multiple_versions
end # find_versions