#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::GobolinuxRecipesComparer
#
# This call can tell us whether we have:
#
#   (a) gobolinux recipes that are not yet registered locally.
#   (b) registered gobolinux recipes having a more recent program version.
#
# Usage example:
#   GobolinuxRecipesComparer.new
# =========================================================================== #
# require 'gobolinux_recipes_comparer.rb'
# =========================================================================== #
require 'open-uri'
require 'cookbooks/aliases/aliases.rb'
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'
require 'cookbooks/module_methods/swift_return_version_of_this_program.rb'

module Cookbooks

class GobolinuxRecipesComparer < Base # === GobolinuxRecipesComparer.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === REMOTE_RECIPES_HOMEPAGE
  # ========================================================================= #
  REMOTE_RECIPES_HOMEPAGE = 'http://recipes.gobolinux.org/r/'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    register_sigint
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === scan_for_remote_versions
  # ========================================================================= #
  def scan_for_remote_versions
    use_this_regex = /<tr><td>\s*<a class='RecipeTitle' href='\?list=(.+?)'>/ # See: http://rubular.com/r/a3dgeZ3gb1
    @scanned_results = @dataset.scan(use_this_regex).flatten
  end

  # ========================================================================= #
  # === read_in_dataset_from_the_remote_webpage
  # ========================================================================= #
  def read_in_dataset_from_the_remote_webpage
    @dataset = open(REMOTE_RECIPES_HOMEPAGE).read
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === iterate_over_the_results
  # ========================================================================= #
  def iterate_over_the_results
    @scanned_results.each_with_index {|line, index| index += 1
      if line.include? '&ver='
        splitted = line.split('&ver=')
        program_name = splitted.first.downcase
        program_version = splitted.last
        if program_version.include?('-r') and
           program_version =~ /\d+$/ # End with a regex.
          end_position = (program_version.index('-r')-1)
          program_version = program_version[0 .. end_position]
        end
        e
        e orange(index)
        e
        if ::Cookbooks.does_include?(program_name, :do_not_use_aliases)
          opnn; e 'The program '+sfancy(program_name)+' is included.'
          opnn; e 'The remote program version is '+simp(program_version)+'.'
          local_program_version = ::Cookbooks.swift_return_version_of_this_program(program_name)
          opnn; e 'The local program version is '+simp(local_program_version)+'.'
          a = gem_version(local_program_version)
          b = gem_version(program_version)
          if a and b
            if a < b
              opnn; e 'The remote version appears to be more up to date than'
              opnn; e 'the local version. This means that you could update.'
            end
          else
            e 'Something went wrong with local version ('+sfancy(local_program_version)+
              ') and program version ('+sfancy(program_version)+').'
          end
        else
          opnn; e 'The program '+sfancy(program_name)+' is NOT included.'
        end
        e
      end
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    read_in_dataset_from_the_remote_webpage
    scan_for_remote_versions
    iterate_over_the_results
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::GobolinuxRecipesComparer.new(ARGV)
end # gobolinux_recipes_comparer