#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::RegisteredLibraries
#
# The task of this class is simple:
#
#   Iterate over all libraries and collect them, and then store
#   them in a yaml file.
# =========================================================================== #
# require 'cookbooks/utility_scripts/registered_libraries.rb'
# =========================================================================== #
require 'cookbooks/utility_scripts/registered.rb'
require 'cookbooks/class/class.rb'
require 'cookbooks/module_methods/all_libraries.rb'

module Cookbooks

class RegisteredLibraries < ::Cookbooks::Registered

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === SHOW_PROGRAM_NAME
  # ========================================================================= #
  SHOW_PROGRAM_NAME = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @hash_to_be_stored = {} # This is the hash that will be stored.
    @query = nil
    @this_program = nil
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === report_to_the_user_what_we_will_do
  # ========================================================================= #
  def report_to_the_user_what_we_will_do
    opnn; e 'Obtaining Information about available libraries next.'
    opnn; e 'This may take a while.'
    opnn; e 'Please be patient.'
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    report_to_the_user_what_we_will_do
    Cookbooks.available_programs?.each { |program_name|
      @query = ::Cookbooks::Cookbook.new(program_name) { :bypass_menu }
      data = @query.return_dataset
      libraries = data['libraries'] # This is an Array.
      unless libraries.empty?
        e program_name if SHOW_PROGRAM_NAME
        libraries.each {|entry|
          @hash_to_be_stored[entry] = program_name
        }
      end
    }
    save_yaml_file
  end

  # ========================================================================= #
  # === save_yaml_file
  #
  # The method `save_yaml_file` will create the yaml file which lists our
  # libraries (in hash format). Right now, we need to manually update this
  # yaml file on our own.
  # ========================================================================= #
  def save_yaml_file
    what = YAML.dump(@hash_to_be_stored)
    into = LOCATION_REGISTERED_LIBRARIES
    e sfancy('Storing yaml dataset in ')
    e "  #{sfile(into)}"
    write_what_into(what, into)
    if is_on_roebe?
      into = "#{RUBY_SRC_DIR_AT_HOME}cookbooks/lib/cookbooks/yaml/"\
             "#{File.basename(LOCATION_REGISTERED_LIBRARIES)}"
      e 'Also storing into:'
      e "  #{sfile(into)}"
      write_what_into(what, into)
    end
  end; alias create_yaml_file save_yaml_file # === create_yaml_file

  # ========================================================================= #
  # === search_for_this_program
  #
  # Use this method to search for a specific program.
  # ========================================================================= #
  def search_for_this_program(
      i = @this_program
    )
    i = i.to_s
    opnn; e "Searching for #{sfancy(i)} now:"
    hash = load_yaml(LOCATION_REGISTERED_LIBRARIES)
    if hash.keys.include? i
      opnn; e "The library called `#{sfancy(i)}` was found."
      opnn; e "It is part of the package -> `#{simp(hash[i])}`."
    else
      opnn; e p+' is '+swarn('NOT')+' included. We could not '\
              'find any program'
      opnn; e 'including this program.'
    end
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _ = Cookbooks::RegisteredLibraries.new(false)
  if ARGV.first
    _.search_for_this_program(ARGV.first)
  else
    _.run
  end
end # reglib