#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::ReportTheRegisteredPrograms
#
# This class will report the registered programs.
#
# It will output a line similar to the following one:
#
#   2845 programs are registered in our cookbooks as of 19.05.2016.
#   2988 programs are registered in the cookbooks as of 20.08.2017.
#
# Usage example:
#   Cookbooks::ReportTheRegisteredPrograms.new
# =========================================================================== #
# require 'cookbooks/utility_scripts/report_the_registered_programs.rb'
# =========================================================================== #
require 'cookbooks/base/prototype.rb'
require 'cookbooks/module_methods/available_programs.rb'

module Cookbooks

class ReportTheRegisteredPrograms < Base # === Cookbooks::ReportTheRegisteredPrograms.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === SHOW_CLASS_NAME
  # ========================================================================= #
  SHOW_CLASS_NAME = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    if commandline_arguments
      unless commandline_arguments.empty?
        case commandline_arguments.first # Use --short if you want to have a shorter variant instead.
        when 'do_not_show_the_class_name','--short'
          @show_class_name = false
        end
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @show_class_name = SHOW_CLASS_NAME
  end

  # ========================================================================= #
  # === show_class_name?
  # ========================================================================= #
  def show_class_name?
    @show_class_name
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === do_report_how_many_programs_are_registered
  # ========================================================================= #
  def do_report_how_many_programs_are_registered
    opnn if show_class_name?
    e sfancy(Cookbooks.n_programs_available?)+' programs are '\
      'registered in the cookbooks as of '+simp(return_date)+'.'
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_report_how_many_programs_are_registered
  end

end

  # ========================================================================= #
  # === Cookbooks.report_the_registered_programs
  # ========================================================================= #
  def self.report_the_registered_programs(i = ARGV)
    Cookbooks::ReportTheRegisteredPrograms.new(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::ReportTheRegisteredPrograms.new(ARGV)
end # nprograms
    # nprograms do_not_show_the_class_name