#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::ShowDependenciesOf
#
# This class will simply show all dependencies of a given program.
#
# This is different to the class ShowDependenciesOn, which shows
# all programs that depend on the given input-program.
#
# Usage examples:
#   sdeps_of ruby
#   sdeps_of libx11
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'

module Cookbooks

class ShowDependenciesOf < Cookbooks::Base # Cookbooks::ShowDependenciesOn

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect.to_s

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      of_this_program = ARGV,
      run_already     = true
    )
    set_of_this_program(of_this_program)
    run if run_already
  end

  # ========================================================================= #
  # === set_of_this_program
  # ========================================================================= #
  def set_of_this_program(of_this_program)
    if of_this_program.is_a? Array
      of_this_program = of_this_program.first
    end
    @of_this_program = of_this_program
  end

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    @of_this_program
  end

  # ========================================================================= #
  # === report_these_dependencies
  # ========================================================================= #
  def report_these_dependencies(deps)
    opnn; e 'The dependencies of `'+sfancy(program?.to_s)+'` are:'; e
    deps.each_with_index {|entry, index| index += 1
      index = index.to_s.rjust(deps.size.to_s.size)
      if Object.const_defined? :Colours
        index = Colours::Konsole.slateblue(index.to_s)
      end
      e '  ('+index.to_s+') '+entry
    }; e
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    _ = Cookbooks::Cookbook.new(program?)
    _.run
    deps = _.dependencies? # This will be an array.
    report_these_dependencies(deps)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::ShowDependenciesOf.new(ARGV)
end # sdof ruby