#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::BatchValidateTheCookbookRecipes
#
# This class will batch-validate the various cookbooks files, by invoking
# the .rb files in this directory. It is thus some kind of "master control"
# file in regards to validation-related queries to the cookbooks recipes.
#
# Usage example:
#   Cookbooks::BatchValidateTheCookbookRecipes.new
# =========================================================================== #
# require 'cookbooks/validation/batch_validate_the_cookbook_recipes.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'

module Cookbooks

class BatchValidateTheCookbookRecipes < Base # === Cookbooks::BatchValidateTheCookbookRecipes.new

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    @array_run_these_files = []
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === run_this_file
  # ========================================================================= #
  def run_this_file(i)
    if i.is_a? Array
      i.each {|entry| run_this_file(entry) }
    else
      # ===================================================================== #
      # We must determine the name of the class. The usual input is
      # something like:
      #   /Programs/Ruby/2.4.2/lib/ruby/site_ruby/2.4.0/cookbooks/validation/scan_for_missing_last_update_entries.rb
      # ===================================================================== #
      load(i)
      _ = File.basename(i).sub(/\.rb$/,'').dup
      _ = to_camelcase(_)
      namespace = 'Cookbooks::'+_
      opnn; e "Now invoking #{slateblue('class '+namespace)}"
      constant = Cookbooks.const_get(_)
      constant.send(:new)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_which_files_to_run
    @array_run_these_files.each {|this_file|
      run_this_file(this_file)
    }
  end

  # ========================================================================= #
  # === determine_which_files_to_run
  # ========================================================================= #
  def determine_which_files_to_run
    @array_run_these_files << Dir["#{Cookbooks::DIRECTORY_VALIDATION}*.rb"]
    @array_run_these_files.flatten!
  end

  # ========================================================================= #
  # === []
  # ========================================================================= #
  def self.[](i = '')
    self.new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::BatchValidateTheCookbookRecipes.new(ARGV)
end # batch_validate_the_cookbook_recipes