#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::CheckForCorrectDependencies
#
# This class will check whether the entries in the cookbook files do all
# happen to have valid, existing dependencies. If the file is not valid,
# the class will output this information on the commandline. That way
# the user can fix the problems.
#
# Usage example:
#   Cookbooks::CheckForCorrectDependencies.new
# =========================================================================== #
# require 'cookbooks/validation/check_for_correct_dependencies.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/class/class.rb'
require 'cookbooks/misc/available_cookbooks.rb'
require 'cookbooks/aliases/aliases.rb'

module Cookbooks

class CheckForCorrectDependencies < Base # === Cookbooks::CheckForCorrectDependencies.new

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = nil)
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === format_program
  # ========================================================================= #
  def format_program(i)
    _ = "#{i}:"
    _ = _.ljust(22)
    splitted = _.split(':')
    splitted[0] = sfancy(splitted.first)
    _ = splitted.join(':')
    _
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    if input?
    else # else check for every entry
      check_for_every_entry
    end
  end

  # ========================================================================= #
  # === check_for_every_entry
  # ========================================================================= #
  def check_for_every_entry
    opnn; e 'Now checking every program. This may take a while.'
    Cookbooks.available_programs?.each {|program|
      # ===================================================================== #
      # The next line is very verbose, so it may be best to enable it only
      # for debugging purposes really.
      # ===================================================================== #
      # opnn; e 'Working on the program `'+sfancy(program)+'` next.'
      dataset = ::Cookbooks::Cookbook.new(program) { :bypass_menu }
      deps    = dataset.deps?
      result  = deps.all? {|entry| Cookbooks.really_does_include?(entry) }
      if result # Ok, in this case all entries were found.
      else
        deps.each {|entry|
          if entry.include? ' '
            entry = entry.split(' ').first.strip
          end
          unless Cookbooks.really_does_include? entry
            opnn; e "#{format_program(program)}No "\
                    "entry for `#{simp(entry)}` was found."
          end
        }
      end
    }
  end

  # ========================================================================= #
  # === Cookbooks::CheckForCorrectDependencies[]
  # ========================================================================= #
  def self.[](i = '')
    self.new
  end

end; end

if __FILE__ == $PROGRAM_NAME
  Cookbooks::CheckForCorrectDependencies.new(ARGV)
end # correct_dependencies