#!/System/Index/bin/ruby -w
# Encoding: ISO-8859-1
# frozen_string_literal: true
# =========================================================================== #
# === Cookbooks::ValidateCookbookAliases
#
# This small class will tell us whether the aliases that are stored in
# cookbook_aliases.yml are valid or whether they are not.
#
# If an entry is not valid then we will report this to the user.
# =========================================================================== #
# require 'cookbooks/validation/validate_cookbook_aliases.rb'
# =========================================================================== #
require 'cookbooks/base/base.rb'
require 'cookbooks/aliases/cookbook_aliases.rb'

module Cookbooks

class ValidateCookbookAliases < Base

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = self.inspect

  # ========================================================================= #
  # === COOKBOOK_ALIASES_FILE
  # ========================================================================= #
  COOKBOOK_ALIASES_FILE = ::Cookbooks.file_cookbook_aliases

  # ========================================================================= #
  # === YAML_COOKBOOK_ALIASES
  # ========================================================================= #
  if File.exist? COOKBOOK_ALIASES_FILE
    YAML_COOKBOOK_ALIASES = YAML.load_file(COOKBOOK_ALIASES_FILE)
  end

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(run_already = true)
    super()
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    @array_this_file = []
    do_not_debug
  end

  # ========================================================================= #
  # === check_against_menu
  # ========================================================================= #
  def check_against_menu(i)
    case i # case tag
    # ======================================================================= #
    # === validate_cookbook_aliases --help
    # ======================================================================= #
    when 'HELP',/-?-?help/
      opnn; e 'You can OPEN the aliases file.'
      exit
    # ======================================================================= #
    # === validate_cookbook_aliases --open
    # ======================================================================= #
    when 'OPEN',/-?-?open/
      opnn; e 'Next opening the file '+sfile(COOKBOOK_ALIASES_FILE)
      esystem 'bluefish '+COOKBOOK_ALIASES_FILE
      exit
    end
  end; alias menu check_against_menu # === menu (menu tag)

  # ========================================================================= #
  # === report_everything_now
  # ========================================================================= #
  def report_everything_now
    @array_this_file.each { |file|
      value = File.basename(file).gsub(/\.yml/,'')
      opnn; ewarn 'Nope, `'+sfile(file)+swarn('` does not exist.')
      opnn; ewarn 'Consider fixing `'+sfancy(value)+swarn('` in order to have a valid '\
                  'cookbook-aliases file.')
    }
    if @array_this_file.empty?
      opnn; e cyan('Everything is fine, all entries are correct!')
    end
  end

  # ========================================================================= #
  # === report_this_lateron
  # ========================================================================= #
  def report_this_lateron(i)
    @array_this_file << i # Append into an Array.
  end; private :report_this_lateron

  # ========================================================================= #
  # === return_sorted
  #
  # Return the aliases in a sorted manner.
  # ========================================================================= #
  def return_sorted
    YAML_COOKBOOK_ALIASES.values.sort
  end

  # ========================================================================= #
  # === find_out_which_files_are_missing
  #
  # This method will find out which cookbook-files are missing.
  # ========================================================================= #
  def find_out_which_files_are_missing
    return_sorted.each {|value|
      # ===================================================================== #
      # The variable value is not allowed to include any '_' character.
      # ===================================================================== #
      value.delete!('_') if value.include? '_'
      # ===================================================================== #
      # Simply do a File.exist? check next.
      # ===================================================================== #
      _ = COOKBOOKS_DIRECTORY+value+'.yml'
      if File.exist?(_)
        # e 'The key `'+key.to_s+'` is pointing at `'+value.to_s+'`. '
        # e 'This is ok, it is a valid entry.'
      else # Does not exist.
        report_this_lateron(_)
      end
    }
  end

  # ========================================================================= #
  # === feedback_debug
  # ========================================================================= #
  def feedback_debug
    opnn; e 'Debug: here is the main dataset:'
    pp return_sorted
  end

  # ========================================================================= #
  # === inform_user_what_will_happen
  # ========================================================================= #
  def inform_user_what_will_happen
    opnn; e 'We will next try to validate the file'
    opnn; e
    opnn; e '  `'+sfile(COOKBOOK_ALIASES_FILE)+'`.'
    opnn; e # Add a newline as well.
  end; private :inform_user_what_will_happen

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(NAMESPACE)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    inform_user_what_will_happen # We inform the user that we validate the cookbook-aliases file.
    find_out_which_files_are_missing
    report_everything_now
    feedback_debug if debug?
  end; alias check_integrity_of_cookbook_aliases run # === check_integrity_of_cookbook_aliases

end; end

if __FILE__ == $PROGRAM_NAME
  _ = Cookbooks::ValidateCookbookAliases.new(false)
  _.menu(ARGV.first) if ARGV.first
  # _.enable_debug
  _.run
end # vca